@@ -1,25 +1,25 @@
-/* 
- * Copyright (C) 2000-2003 the xine project
- * 
+/*
+ * Copyright (C) 2000-2004 the xine project
+ *
  * This file is part of xine, a free video player.
- * 
+ *
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * xine is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: plugin.c,v 1.22 2003/04/08 22:07:47 guenter Exp $
+ * $Id: plugin.c,v 1.28 2004/11/11 01:22:52 dsalt Exp $
  *
- * xine plugin for mozilla 
+ * xine plugin for mozilla
  *
  */
 
@@ -30,6 +30,7 @@
 #include <stdlib.h>
 #include <dlfcn.h>
 #include <errno.h>
+#include <signal.h>
 #include <X11/X.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -47,7 +48,7 @@
 
 #include <stdio.h>
 #include <math.h>
-
+#include <unistd.h>
 
 
 #define XP_UNIX
@@ -61,6 +62,10 @@
 #define LOG
 */
 
+/* FIXME: browser exit on URL box destroy, segfault on 'close' click
+#define URLBOX
+*/
+
 /*
  * what player do we try to emulate?
  */
@@ -92,7 +97,7 @@ typedef struct {
   Window                window;
   Widget                top_level;
   int                   width,height;
-  
+
   int                   emu_mode;
 
   char                 *real_controls;
@@ -118,16 +123,16 @@ static void xprintf (const char *format, ...) {
     log_fd = fopen ("/tmp/t.log", "a+");
 
     if (log_fd) {
-      
+
       setvbuf (log_fd, NULL, _IONBF, 0);
 
       fprintf (log_fd, "\n---------------------------------------------\n\n");
-      
+
     }
   }
 
   va_start (argp, format);
-    
+
   if (log_fd) vfprintf (log_fd, format, argp);
 
   va_end (argp);
@@ -135,9 +140,9 @@ static void xprintf (const char *format, ...) {
 }
 
 char *NPP_GetMIMEDescription(void) {
-  
+
   xprintf("NPP_GetMIMEDescription:\n");
-  
+
   return "video/mpeg: mpeg, mpg, mpe: MPEG animation;"
          "video/x-mpeg: mpeg, mpg, mpe: MPEG animation;"
          "audio/mpeg2: mp2: MPEG audio;"
@@ -206,19 +211,19 @@ static void print_status (plugin_instance_t *this, const char *format, ...) {
   va_list argp;
 
   va_start (argp, format);
-    
+
   vsnprintf (this->status_str, STATUS_STR_SIZE, format, argp);
 
   va_end (argp);
-  
+
 #if 0
   paint_it (this);
 #endif
 }
 
 /* fork2() -- like fork, but the new process is immediately orphaned
- *            (won't leave a zombie when it exits)                 
- * Returns 1 to the parent, not any meaningful pid.               
+ *            (won't leave a zombie when it exits)
+ * Returns 1 to the parent, not any meaningful pid.
  * The parent cannot wait() for the new process (it's unrelated).
  */
 
@@ -228,7 +233,7 @@ static void print_status (plugin_instance_t *this, const char *format, ...) {
 static int fork2() {
   pid_t pid;
   int status;
-  
+
   sigset_t set,oset;
   sigfillset(& set);
   xprintf (">>>>>>>>Forking<<<<<<<<,\n");
@@ -237,19 +242,19 @@ static int fork2() {
   if (!(pid = fork())) {
     xprintf ("child\n");
     switch (fork()) {
-    case 0:    
+    case 0:
       xprintf ("child 2\n");
       sigprocmask(SIG_SETMASK,&oset,&set);
       return 0;
-    case -1: 
+    case -1:
       xprintf ("fork 2 failed!\n");
       _exit(errno);    /* assumes all errnos are <256 */
-    default: 
+    default:
       xprintf ("parent 2\n");
       _exit(0);
     }
   }
-  
+
   xprintf ("parent, child pid =%d\n", pid);
 
   if (pid < 0 || waitpid(pid,&status,0) < 0) {
@@ -258,7 +263,7 @@ static int fork2() {
     return -1;
   }
   sigprocmask(SIG_SETMASK,&oset,&set);
-  
+
   xprintf ("waitpid done\n");
 
   if (WIFEXITED(status))
@@ -269,7 +274,7 @@ static int fork2() {
       errno = WEXITSTATUS(status);
   else
     errno = EINTR;  /* well, sort of :-) */
-  
+
   xprintf ("parent done\n");
 
   return -1;
@@ -278,35 +283,39 @@ static int fork2() {
 static void launch_gxine (plugin_instance_t *this) {
 
   if (!globals.url) {
-    xprintf ("launch_gxine: no url!\n");
+    printf ("launch_gxine: no url!\n");
     return;
   }
 
   if (!fork2()) {
-    char cmd [4096];
-    
+    char executable[4096];
+    char arg[4096];
+
+    snprintf(executable, 4096, "%s/gxine", GXINE_BINDIR);
+
     if (this->emu_mode == EMU_MPLAYER)
-      snprintf (cmd, 4096, "%s/gxine 'mms%s'", GXINE_BINDIR, globals.url);
+      snprintf(arg, 4096, "mms%s", globals.url);
     else
-      snprintf (cmd, 4096, "%s/gxine '%s'", GXINE_BINDIR, globals.url);
-    
-    xprintf ("launch_gxine: system(%s)\n", cmd);
-    
-    system (cmd);
-    
-    _exit (0);
+      strncpy(arg, globals.url, 4096);
+
+    xprintf ("launch_gxine: exe = %s, arg = %s\n", executable, arg);
+
+    if (execlp("gxine", executable, arg, NULL) == -1) {
+       perror("Error while launching gxine");
+       _exit(0);
+    }
   }
-  
+
   xprintf ("gxine launched.\n");
 
 #if 0
   if (this->gtk_wind) {
     print_status (this, "gxine launched");
-  
+
     xprintf ("gxine launched\n");
-    
+
     gtk_entry_set_text (GTK_ENTRY (this->mrl_entry), globals.url);
-    
+
     xprintf ("launch_gxine: done\n");
   }
 #endif
@@ -334,9 +343,9 @@ NPError NPP_New(NPMIMEType pluginType, NPP instance,
 
   if (instance == NULL)
     return NPERR_INVALID_INSTANCE_ERROR;
-  
+
   instance->pdata = NPN_MemAlloc(sizeof(plugin_instance_t));
-  
+
   this = (plugin_instance_t*) instance->pdata;
 
   globals.url           = NULL;
@@ -350,7 +359,7 @@ NPError NPP_New(NPMIMEType pluginType, NPP instance,
     /* parse args */
 
     this->emu_mode = EMU_NONE;
-    
+
     for (i=0; i<argc; i++) {
 
      xprintf ("plugin: argument '%s'='%s'\n",
@@ -363,13 +372,13 @@ NPError NPP_New(NPMIMEType pluginType, NPP instance,
 	} else if (!strncmp (argv[i], "application/x-mplayer2", 22)) {
 	  xprintf ("plugin: switching to mms_mode\n");
 	  this->emu_mode = EMU_MPLAYER;
-	} else if (!strncmp (argv[i], "video/quicktime", 21)) {
+	} else if (!strncmp (argv[i], "video/quicktime", 15)) {
 	  xprintf ("plugin: switching to quicktime emulation mode\n");
 	  this->emu_mode = EMU_QT;
 	} else if (!strncmp (argv[i], "audio/x-pn-realaudio-plugin", 27)) {
 	  xprintf ("plugin: switching to real player emulation mode\n");
 	  this->emu_mode = EMU_REAL;
-	} 
+	}
       } else if (!strcmp (argn[i], "name")) {
 	if (!strcmp (argv[i], "nsplay")) {
 	  xprintf ("plugin: switching to mms_mode\n");
@@ -381,19 +390,19 @@ NPError NPP_New(NPMIMEType pluginType, NPP instance,
       } else if (!strcasecmp (argn[i], "src") && !globals.url) {
 	got_url (argv[i]);
 	xprintf ("got src url %s\n", globals.url);
-      } else if (!strcasecmp (argn[i], "controls") && (this->emu_mode = EMU_REAL)) {
+      } else if (!strcasecmp (argn[i], "controls") && (this->emu_mode == EMU_REAL)) {
 	this->real_controls = strdup (argv[i]);
 	xprintf ("got controls %s\n", this->real_controls);
-      } else if (!strcasecmp (argn[i], "autostart") && (this->emu_mode = EMU_REAL)) {
+      } else if (!strcasecmp (argn[i], "autostart") && (this->emu_mode == EMU_REAL)) {
 	this->autostart = !strcasecmp (argv[i], "true");
 	xprintf ("got autostart %d\n", this->autostart);
-      } 
+      }
 
     }
 
-    if ( (this->emu_mode == EMU_REAL) 
+    if ( (this->emu_mode == EMU_REAL)
 	 && this->autostart && globals.url
-	 && !globals.gxine_started) 
+	 && !globals.gxine_started)
       launch_gxine (this);
 
     xprintf ("plugin: NPP_New done\n");
@@ -406,7 +415,7 @@ NPError NPP_New(NPMIMEType pluginType, NPP instance,
   }
 }
 
-void play_cb (Widget w, XtPointer client_data, XtPointer call_data) {
+static void play_cb (Widget w, XtPointer client_data, XtPointer call_data) {
 
   plugin_instance_t *this = (plugin_instance_t *) client_data;
 
@@ -415,13 +424,14 @@ void play_cb (Widget w, XtPointer client_data, XtPointer call_data) {
   launch_gxine (this);
 }
 
-void close_cb (Widget w, XtPointer client_data, XtPointer call_data) {
+static void close_cb (Widget w, XtPointer client_data, XtPointer call_data) {
   Widget shell = (Widget) client_data;
 
   XtPopdown (shell);
 }
 
-void url_cb (Widget w, XtPointer client_data, XtPointer call_data) {
+#ifdef URLBOX
+static void url_cb (Widget w, XtPointer client_data, XtPointer call_data) {
 
   plugin_instance_t *this = (plugin_instance_t *) client_data;
   Widget  shell, box, entry, button;
@@ -435,16 +445,16 @@ void url_cb (Widget w, XtPointer client_data, XtPointer call_data) {
   xprintf ("popup\n");
 
   shell = XtVaCreatePopupShell ("url popup", transientShellWidgetClass,
-				this->top_level, 
+				this->top_level,
 				XtNpopdownCallback, XtCallbackNone, NULL);
 
-  box = XtVaCreateManagedWidget ("box", boxWidgetClass, shell, 
+  box = XtVaCreateManagedWidget ("box", boxWidgetClass, shell,
 				 NULL);
-  
-  entry = XtVaCreateManagedWidget ("entry", asciiTextWidgetClass, box, 
+
+  entry = XtVaCreateManagedWidget ("entry", asciiTextWidgetClass, box,
 				   XtNstring, globals.url,
 				   XtNdisplayCaret, False,
-				   XtNresize, XawtextResizeBoth, 
+				   XtNresize, XawtextResizeBoth,
 				   XtNwidth, 340, NULL);
 
   button = XtVaCreateManagedWidget ("close", commandWidgetClass, box, NULL);
@@ -452,6 +462,7 @@ void url_cb (Widget w, XtPointer client_data, XtPointer call_data) {
 
   XtPopup (shell, XtGrabNone);
 }
+#endif
 
 NPError NPP_SetWindow(NPP instance, NPWindow* window) {
 
@@ -459,19 +470,19 @@ NPError NPP_SetWindow(NPP instance, NPWindow* window) {
   Widget hello, button, box;
 
   xprintf("NPP_SetWindow: 42\n");
- 
+
   if (instance == NULL) {
    xprintf("NPERR_INVALID_INSTANCE_ERROR\n");
     return NPERR_INVALID_INSTANCE_ERROR;
   }
-  
+
   if (window == NULL) {
     xprintf("window == NULL, NPERR_NO_ERROR\n");
     return NPERR_NO_ERROR;
   }
-  
+
   this = (plugin_instance_t*) instance->pdata;
-  
+
   this->display   = ((NPSetWindowCallbackStruct *) window->ws_info)->display;
   this->window    = (Window) window->window;
   this->width     = window->width;
@@ -491,32 +502,50 @@ NPError NPP_SetWindow(NPP instance, NPWindow* window) {
   XSync (this->display, FALSE);
 
 
-  box = XtVaCreateManagedWidget ("form", formWidgetClass, this->top_level, 
-				 XtNbackground, this->black, 
-				 XtNwidth, this->width, 
+  box = XtVaCreateManagedWidget ("form", formWidgetClass, this->top_level,
+				 XtNbackground, this->black,
+				 XtNwidth, this->width,
 				 XtNheight, this->height, NULL);
 
-  hello = XtVaCreateManagedWidget ("gxine browser plugin", labelWidgetClass, box, 
-				   XtNbackground, this->black, 
-				   XtNforeground, this->white, 
-				   XtNwidth, this->width, 
+  hello = XtVaCreateManagedWidget ("gxine browser plugin", labelWidgetClass, box,
+				   XtNbackground, this->black,
+				   XtNforeground, this->white,
+				   XtNwidth, this->width,
 				   XtNheight, this->height, NULL);
   if (this->real_controls && !strcasecmp (this->real_controls, "PlayonlyButton")) {
-    button = XtVaCreateManagedWidget (">", commandWidgetClass, box, 
-				      XtNbackground, this->black, 
-				      XtNforeground, this->white, 
+    button = XtVaCreateManagedWidget (">", commandWidgetClass, box,
+				      XtNbackground, this->black,
+				      XtNforeground, this->white,
 				      XtNborderColor, this->white, NULL);
     XtAddCallback (button, XtNcallback, play_cb, this);
   } else {
-    button = XtVaCreateManagedWidget ("url", commandWidgetClass, box, 
-				      XtNbackground, this->black, 
-				      XtNforeground, this->white, 
-				      XtNborderColor, this->white, NULL);
-    XtAddCallback (button, XtNcallback, url_cb, this);
+#ifdef URLBOX
+     button = XtVaCreateManagedWidget ("url", commandWidgetClass, box,
+ 				      XtNbackground, this->black,
+ 				      XtNforeground, this->white,
+ 				      XtNborderColor, this->white, NULL);
+     XtAddCallback (button, XtNcallback, url_cb, this);
+#else
+    Pixel grey = (((this->black     & 0xFF)*3 + (this->white     & 0xFF)) / 4)     |
+		 (((this->black>> 8 & 0xFF)*3 + (this->white>> 8 & 0xFF)) / 4)<< 8 |
+		 (((this->black>>16 & 0xFF)*3 + (this->white>>16 & 0xFF)) / 4)<< 16|
+		 (((this->black>>24 & 0xFF)*3 + (this->white>>24 & 0xFF)) / 4)<< 24;
+    XtVaCreateManagedWidget ("url", asciiTextWidgetClass, box,
+			     XtNstring, globals.url,
+			     XtNdisplayCaret, False,
+			     XtNresize, XawtextResizeBoth,
+			     XtNwidth, this->width,
+			     XtNscrollHorizontal, XawtextScrollWhenNeeded,
+			     XtNscrollVertical, XawtextScrollWhenNeeded,
+			     XtNwrap, XawtextWrapLine,
+			     XtNbackground, grey, //this->black,
+			     XtNforeground, this->white,
+			     XtNborderWidth, 0,
+			     NULL);
+#endif
   }
   XtRealizeWidget (box);
 
-
   xprintf("NPP_SetWindow: done.\n");
 
   return NPERR_NO_ERROR;
@@ -533,23 +562,23 @@ NPError NPP_Destroy(NPP instance, NPSavedData** save) {
   this = (plugin_instance_t*) instance->pdata;
 
   if (this != NULL) {
-    
+
     NPN_MemFree(instance->pdata);
     instance->pdata = NULL;
-    
+
   }
-  
+
   globals.gxine_started = FALSE;
 
   xprintf("NPP_Destroy: closed.\n");
-  
+
   return NPERR_NO_ERROR;
 }
 
 
 NPError NPP_NewStream (NPP instance,
 		       NPMIMEType type,
-		       NPStream *stream, 
+		       NPStream *stream,
 		       NPBool seekable,
 		       uint16 *stype) {
 
@@ -560,7 +589,7 @@ NPError NPP_NewStream (NPP instance,
 
   if (instance == NULL)
     return NPERR_INVALID_INSTANCE_ERROR;
-  
+
   this = (plugin_instance_t*) instance->pdata;
 
   xprintf("NPP_NewStream: url is %s \n", stream->url);
@@ -580,7 +609,7 @@ NPError NPP_NewStream (NPP instance,
        *
        * poor gxine :>
        */
-      
+
       launch_gxine (this);
       xprintf ("NPP_NewStream: gxine started successfully\n");
     }
@@ -663,17 +692,17 @@ void NPP_Print(NPP instance, NPPrint* printInfo) {
   /*
     if (instance != NULL) {
     plugin_instance_t* this = (plugin_instance_t*) instance->pdata;
-	
+
     if (printInfo->mode == NP_FULL) {
 
     void* platformPrint =
     printInfo->print.fullPrint.platformPrint;
     NPBool printOne =
     printInfo->print.fullPrint.printOne;
-			
+
     printInfo->print.fullPrint.pluginPrinted = FALSE;
 
-    } else {	
+    } else {
 
     NPWindow* printWindow =
     &(printInfo->print.embedPrint.window);
