@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002 the xine-project
+ * Copyright (C) 2002-2005 the xine-project
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -16,7 +16,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
  *
- * $Id: desktop_integration.c,v 1.7 2004/11/10 01:29:12 dsalt Exp $
+ * $Id: desktop_integration.c,v 1.8.2.2 2005/07/14 23:46:17 dsalt Exp $
  *
  * helper functions to register gxine with kde and gnome
  * (and possibly cde in the future)
@@ -270,7 +270,7 @@ void gnome_vfs_shutdown() {
     void (* gnome_vfs_application_registry_sync) ( void ) = NULL;
     
     /* extract and call shutdown functions */
-    gnome_vfs_mime_info_shutdown = (void(*)()) dlsym( gnome_vfs_handle, "gnome_vfs_mime_info_shutdown" );
+    gnome_vfs_mime_info_shutdown = (void(*)()) dlsym( gnome_vfs_handle, "gnome_vfs_mime_shutdown" );
 
     if( NULL == gnome_vfs_mime_info_shutdown )  {
       fprintf( stderr, "%s.\n", dlerror() );
@@ -409,16 +409,20 @@ void di_register_kde (void) {
   }
 
   fprintf (f, "[Desktop Entry]\n"
-	   "Version=0.92\n"
+	   "Version=1.0\n"
 	   "Encoding=UTF-8\n"
 	   "Name=gxine\n"
+	   "GenericName=Video Player\n"
+ 	   "Comment=Play films and songs, or watch digital TV\n"
 	   "MultipleArgs=true\n"
 	   "Terminal=0\n"
-	   "Icon=%s/gxine-icon.xpm\n"
+	   "Icon=%s/gxine-logo.png\n"
 	   "Exec=%s/gxine\n"
 	   "Type=Application\n"
+ 	   "StartupNotify=true\n"
+ 	   "Categories=GTK;Application;AudioVideo;\n"
 	   "MimeType=",
-	   pixmapdir,
+	   icondir,
 	   bindir);
 
   /*
