@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003 the xine-project
+ * Copyright (C) 2003-2005 the xine-project
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -16,7 +16,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
  *
- * $Id: drag_drop.h,v 1.1 2003/02/01 23:30:06 guenter Exp $
+ * $Id: drag_drop.h,v 1.2 2005/01/31 19:04:45 dsalt Exp $
  *
  * drag and drop support functions
  */
@@ -24,6 +24,22 @@
 #ifndef HAVE_DRAG_DROP_H
 #define HAVE_DRAG_DROP_H
 
-void drag_drop_setup (GtkWidget *widget) ;
+typedef struct {
+  GtkTreeModel *model;
+  GtkTreePath *path;
+  int index;
+} drag_mrl_t;
+
+extern drag_mrl_t drag_mrl;
+
+int dnd_add_mrls (GtkSelectionData *, int *current, int ins_pos);
+
+void treeview_drag_drop_setup (GtkTreeView *,
+			       void (*drop_cb) (GtkTreeView *, GdkDragContext *,
+						gint, gint, GtkSelectionData *,
+						guint, guint));
+int treeview_get_drop_index (GtkTreeView *, GdkDragContext *, gint x, gint y);
+
+void drag_drop_setup (GtkWidget *);
 
 #endif
