@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2004 the xine project
+ * Copyright (C) 2001-2005 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: gtkvideo.c,v 1.26 2004/12/16 00:36:55 dsalt Exp $
+ * $Id: gtkvideo.c,v 1.41.2.3 2005/06/28 18:07:26 dsalt Exp $
  *
  * gtk xine video widget
  *
@@ -53,6 +53,7 @@
 #include <xine.h>
 
 #include "gtkvideo.h"
+#include "post.h"
 
 /* missing stuff from libxine includes (<= 1-rc7) */
 #ifndef XINE_GUI_SEND_WILL_DESTROY_DRAWABLE
@@ -79,6 +80,7 @@ static void gtk_video_size_request (GtkWidget      *widget,
 static void gtk_video_size_allocate (GtkWidget      *widget,
  				     GtkAllocation  *allocation);
 
+
 static GtkWidgetClass *parent_class = NULL;
 static pthread_mutex_t resize_lock = PTHREAD_MUTEX_INITIALIZER;
 
@@ -105,6 +107,8 @@ struct gtk_video_private_s {
   int                      xpos, ypos;
   float                    resize_factor;
   gboolean                 resize, auto_resize, allow_resize, force_resize;
+  gboolean		   resizing;
+  gboolean		   auto_rescale;
   int                      video_width, video_height;
   int                      oldwidth, oldheight;
 
@@ -120,6 +124,9 @@ struct gtk_video_private_s {
   Cursor                   no_cursor;
   Cursor                   on_cursor;
   gboolean                 cursor_visible;
+  gboolean		   mapped;
+  pthread_mutex_t	   map_lock;
+  pthread_cond_t	   map_cond;
 
   Bool                   have_xtest;
 #ifdef HAVE_XTESTEXTENSION
@@ -131,6 +138,14 @@ struct gtk_video_private_s {
   xine_post_t		  *vis_plugin;
   char			  *vis_plugin_id;
   xine_audio_port_t       *vis_audio;
+
+  /* tvtime */
+  xine_post_t		  *tvtime_plugin;
+  gboolean		   deinterlace;
+
+  /* other post-plugins (lists of xine_post_t) */
+  GList			  *post_video, *post_audio;
+  gboolean		   post_video_enable, post_audio_enable;
 };
 
 static int gtv_table_signals[LAST_SIGNAL] = { 0 };
@@ -216,6 +231,13 @@ static void gtk_video_finalize (GObject *object) {
 }
 
 
+#define DOUBLE(RF,W,H) \
+  ((RF) * (1 + (priv->auto_rescale \
+	   && (W) <= priv->fullscreen_width / 3.0 \
+	   && (H) <= priv->fullscreen_height / 3.0)))
+#define DOUBLE_STD() \
+  (DOUBLE(priv->resize_factor, priv->video_width, priv->video_height))
+
 static void dest_size_cb (void *gv_gen,
 			  int video_width, int video_height,
 			  double video_pixel_aspect,
@@ -226,10 +248,12 @@ static void dest_size_cb (void *gv_gen,
   gtk_video_private_t *priv = gv->priv;
 
   /* correct size with video_pixel_aspect */
+/*
   if (video_pixel_aspect >= priv->display_ratio)
     video_width  = video_width * video_pixel_aspect / priv->display_ratio + .5;
   else
     video_height = video_height * priv->display_ratio / video_pixel_aspect + .5;
+ */
 
   if (priv->fullscreen_mode) {
     *dest_width  = priv->fullscreen_width;
@@ -247,8 +271,10 @@ static gboolean gtk_video_idle_resize (GtkVideo *gtv) {
   GtkWindow           *toplevel;
   GtkRequisition       toplevel_size;
   gtk_video_private_t *priv = gtv->priv;
-  int                  video_width  = priv->video_width * priv->resize_factor;
-  int                  video_height = priv->video_height * priv->resize_factor;
+
+  double factor = DOUBLE_STD();
+  int video_width  = priv->video_width * factor;
+  int video_height = priv->video_height * factor;
 
   logprintf ("gtkvideo: idle resize to %d x %d\n", video_width, video_height);
 
@@ -283,6 +309,7 @@ static gboolean gtk_video_idle_resize (GtkVideo *gtv) {
   priv->resize = priv->auto_resize;
   priv->force_resize = FALSE;
   priv->allow_resize = TRUE;
+  priv->resizing = FALSE;
 
   logprintf ("gtkvideo: idle signal done\n");
 
@@ -320,8 +347,9 @@ static void frame_output_cb (void *gv_gen,
   /* process windowed mode resize even if we're in full-screen mode */
   if (priv->resize || (priv->allow_resize && priv->force_resize))
   {
-    video_width  *= priv->resize_factor;
-    video_height *= priv->resize_factor;
+    double factor = DOUBLE(priv->resize_factor, video_width, video_height);
+    video_width  *= factor;
+    video_height *= factor;
 
     /* size changed? */
     if (video_width != priv->oldwidth || video_height != priv->oldheight ||
@@ -333,9 +361,13 @@ static void frame_output_cb (void *gv_gen,
       pthread_mutex_lock (&resize_lock);
       priv->oldwidth = video_width;
       priv->oldheight = video_height;
-      pthread_mutex_unlock (&resize_lock);
 
-      g_idle_add ((GSourceFunc) gtk_video_idle_resize, gtv);
+      if (!priv->resizing)
+      {
+        priv->resizing = TRUE;
+        g_idle_add ((GSourceFunc) gtk_video_idle_resize, gtv);
+      }
+      pthread_mutex_unlock (&resize_lock);
     }
   }
 
@@ -507,6 +539,17 @@ static void* xine_thread (void *this_gen) {
 			       XINE_GUI_SEND_EXPOSE_EVENT, &event);
       break;
 
+    case MapNotify:
+      pthread_mutex_lock (&priv->map_lock);
+      priv->mapped = TRUE;
+      pthread_cond_signal (&priv->map_cond);
+      pthread_mutex_unlock (&priv->map_lock);
+      break;
+
+    case UnmapNotify:
+      priv->mapped = FALSE;
+      break;
+
     case FocusOut: /* when in fullscreen mode, keep the focus */
       sched_yield ();
       if (priv->fullscreen_mode && !have_focus) {
@@ -517,7 +560,7 @@ static void* xine_thread (void *this_gen) {
       }
       break;
 
-    case MotionNotify: 
+    case MotionNotify:
       {
 	XMotionEvent     *mevent = (XMotionEvent *) &event;
 	x11_rectangle_t   rect;
@@ -612,7 +655,7 @@ static void* xine_thread (void *this_gen) {
        * the corresponding press event was received
        */
       logprintf("gtkvideo: mouse button release: mx=%d my=%d\n",
-		bevent->x, bevent->y);
+		event.xbutton.x, event.xbutton.y);
 
       if (priv->button_event_shifted & (1 << event.xbutton.button))
 	priv->button_event_shifted &= ~(1 << event.xbutton.button);
@@ -724,21 +767,14 @@ typedef struct {
   uint32_t                    status;
 } MWMHints;
 
-static void create_fullscreen_window (gtk_video_private_t *priv)
+static void setup_fullscreen_window (gtk_video_private_t *priv)
 {
   static char *window_title = N_("gxine fullscreen window");
   XSizeHints   hint;
   Atom         prop;
+  long	       propvalue[3];
   MWMHints     mwmhints;
 
-  int screen = DefaultScreen (priv->display);
-
-  priv->fullscreen_window =
-    XCreateSimpleWindow (priv->display, RootWindow (priv->display, screen),
-			 0, 0, priv->fullscreen_width, priv->fullscreen_height,
-			 1, BlackPixel(priv->display, screen),
-			 BlackPixel(priv->display, screen));
-
   hint.flags       = PPosition | PSize | PMinSize | PWinGravity;
   hint.x           = 0;
   hint.y           = 0;
@@ -753,39 +789,33 @@ static void create_fullscreen_window (gtk_video_private_t *priv)
 			  gettext (window_title), None, NULL, 0, &hint);
   XSetWMNormalHints (priv->display, priv->fullscreen_window, &hint);
 
-  /* XSetWMHints (priv->display, priv->fullscreen_window, &hint);*/
-
   /* layer above most other things, like gnome panel */
-  {
-    static Atom XA_WIN_LAYER = None;
-    long        propvalue[1];
+  prop = XInternAtom(priv->display, "_WIN_LAYER", False);
+  propvalue[0] = 10; /* WIN_LAYER_ABOVE_DOCK */
+  XChangeProperty(priv->display, priv->fullscreen_window, prop, XA_CARDINAL, 32,
+		  PropModeReplace, (unsigned char *)propvalue, 1);
 
-    if (XA_WIN_LAYER == None)
-      XA_WIN_LAYER = XInternAtom(priv->display, "_WIN_LAYER", False);
+  /* fullscreen and stickiness the modern (e.g. metacity) way */
+  /* (do we want stickiness? - Debian bug 285984) */
+  prop = XInternAtom (priv->display, "_NET_WM_STATE", False);
+  propvalue[0] = XInternAtom (priv->display, "_NET_WM_STATE_FULLSCREEN", False);
+  propvalue[1] = XInternAtom (priv->display, "_NET_WM_STATE_STICKY", False);
+  propvalue[2] = 0;
+  XChangeProperty (priv->display, priv->fullscreen_window, prop, XA_ATOM, 32,
+		   PropModeReplace, (unsigned char *)propvalue, 2);
 
-    propvalue[0] = 10; /* WIN_LAYER_ABOVE_DOCK */
-    XChangeProperty(priv->display, priv->fullscreen_window,
-		    XA_WIN_LAYER, XA_CARDINAL, 32, PropModeReplace,
-		    (unsigned char *)propvalue, 1);
-  }
+  prop = XInternAtom (priv->display, "_NET_WM_ALLOWED_ACTIONS", False);
+  propvalue[0] = 0;
+  XChangeProperty (priv->display, priv->fullscreen_window, prop, XA_ATOM, 32,
+		   PropModeReplace, (unsigned char *)propvalue, 0);
 
-  /* fullscreen the modern (e.g. metacity) way */
-  {
-    static Atom XA_WIN_STATE = None;
-    long        propvalue[2];
+  /* and put it on all desktops (do we want this? - Debian bug 285984)) */
+  prop = XInternAtom(priv->display, "_NET_WM_DESKTOP", False);
+  propvalue[0] = -1; /* WIN_LAYER_ABOVE_DOCK */
+  XChangeProperty(priv->display, priv->fullscreen_window, prop, XA_CARDINAL, 32,
+		  PropModeReplace, (unsigned char *)propvalue, 1);
 
-    if (XA_WIN_STATE == None)
-      XA_WIN_STATE = XInternAtom (priv->display, "_NET_WM_STATE", False);
-
-    propvalue[0] = XInternAtom (priv->display, "_NET_WM_STATE_FULLSCREEN",
-				False);
-    propvalue[1] = 0;
-
-    XChangeProperty (priv->display, priv->fullscreen_window,
-		     XA_WIN_STATE, XA_ATOM, 32, PropModeReplace,
-		     (unsigned char *)propvalue, 1);
-    XFlush(priv->display);
-  }
+  XFlush(priv->display);
 
   /* wm, no borders please */
   prop = XInternAtom(priv->display, "_MOTIF_WM_HINTS", False);
@@ -798,6 +828,19 @@ static void create_fullscreen_window (gtk_video_private_t *priv)
   XSetTransientForHint(priv->display, priv->fullscreen_window, None);
 }
 
+void gtk_video_grab_focus (GtkVideo *gtv)
+{
+  gtk_video_private_t *priv;
+
+  g_return_if_fail (gtv != NULL);
+  g_return_if_fail (GTK_IS_VIDEO (gtv));
+
+  priv = gtv->priv;
+  if (priv->fullscreen_mode)
+    XSetInputFocus (priv->display, priv->fullscreen_window,
+		    RevertToNone, CurrentTime);
+}
+
 static void gtk_video_realize (GtkWidget *widget)
 {
   /* GdkWindowAttr attributes; */
@@ -820,11 +863,6 @@ static void gtk_video_realize (GtkWidget *widget)
   /* set realized flag */
   GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
 
-  if (!XInitThreads ()) {
-    printf (_("gtkvideo: XInitThreads failed - looks like you don't have a thread-safe xlib.\n"));
-    return;
-  }
-
   priv->display = XOpenDisplay (NULL);
 
   if (!priv->display) {
@@ -844,7 +882,8 @@ static void gtk_video_realize (GtkWidget *widget)
 
   priv->video_window = XCreateSimpleWindow (gdk_display,
 					    GDK_WINDOW_XWINDOW (gtk_widget_get_parent_window(widget)),
-					    0, 0,
+					    widget->allocation.x,
+					    widget->allocation.y,
 					    widget->allocation.width,
 					    widget->allocation.height, 1,
 					    black_pixel, black_pixel);
@@ -865,7 +904,12 @@ static void gtk_video_realize (GtkWidget *widget)
    * create our fullscreen window
    */
 
-  create_fullscreen_window (priv);
+  priv->fullscreen_window =
+    XCreateSimpleWindow (priv->display,
+			 RootWindow (priv->display, priv->screen),
+			 0, 0, priv->fullscreen_width, priv->fullscreen_height,
+			 1, BlackPixel(priv->display, priv->screen),
+			 BlackPixel(priv->display, priv->screen));
 
   /*
    * track configure events of toplevel window
@@ -907,7 +951,8 @@ static void gtk_video_realize (GtkWidget *widget)
     return ;
   }
 
-  xine_post_wire_video_port (priv->out, priv->video_port);
+  priv->tvtime_plugin = xine_post_init (priv->xine, "tvtime", 0,
+					NULL, &priv->video_port);
 
   /*
    * create mouse cursors
@@ -998,27 +1043,29 @@ GtkWidget *gtk_video_new (xine_t *xine, xine_stream_t *stream,
   gtv->priv = priv = malloc (sizeof (gtk_video_private_t));
   memset (priv, 0, sizeof (gtk_video_private_t));
 
-  if (video_driver_id)
-    priv->video_driver_id = strdup(video_driver_id);
-  else
-    priv->video_driver_id = NULL;
-
   priv->xine = xine;
   priv->stream = stream;
   priv->out = out;
   priv->resize_factor = 0.5;
-  priv->resize = FALSE;
-  priv->allow_resize = FALSE;
   priv->auto_resize = TRUE;
   priv->video_width = default_width;
   priv->video_height = default_height;
   priv->oldwidth = default_width;
   priv->oldheight = default_height;
+  priv->map_lock = (pthread_mutex_t)PTHREAD_MUTEX_INITIALIZER;
+  priv->map_cond = (pthread_cond_t)PTHREAD_COND_INITIALIZER;
   priv->button_press_mask = button_press_mask;
   priv->button_release_mask = button_release_mask;
-  priv->vis_active = FALSE;
   priv->vis_plugin_id = NULL;
   priv->vis_plugin = NULL;
+  priv->tvtime_plugin = NULL;
+  priv->post_audio = priv->post_video = NULL;
+
+  if (video_driver_id)
+    priv->video_driver_id = strdup (video_driver_id);
+  else
+    priv->video_driver_id = NULL;
+
   return this;
 }
 
@@ -1080,22 +1127,31 @@ void gtk_video_resize (GtkVideo *gtv,
 
 void gtk_video_rescale (GtkVideo *gtv, float scale)
 {
-  GtkRequisition size;
-  GtkWidget *toplevel = gtk_widget_get_toplevel (&gtv->widget);
-  gint w = gtv->priv->video_width * scale + .5;
-  gint h = gtv->priv->video_height * scale + .5;
+  gtk_video_private_t *priv = gtv->priv;
+  double factor = DOUBLE(scale, priv->video_width, priv->video_height);
+  gint w = priv->video_width * factor + .5;
+  gint h = priv->video_height * factor + .5;
+  gboolean real = GTK_WIDGET_REALIZED(&gtv->widget);
 
-  XLockDisplay (gtv->priv->display);
-  gtv->priv->resize = gtv->priv->force_resize = FALSE;
-  gtv->priv->resize_factor = scale;
-  /* set the widget's size now so that the window's size is recalculated... */
+  if (real)
+    XLockDisplay (priv->display);
+  priv->resize = priv->force_resize = FALSE;
+  priv->resize_factor = scale;
+  /* set the widget's size now so that the container's size is recalculated */
   gtk_widget_set_size_request (&gtv->widget, w, h);
-  /* ... and get the recalculated size (we want the width) */
-  gtk_widget_size_request (toplevel, &size);
-  gtv->widget.allocation.width = w > size.width ? w : size.width;
+  if (real && gtv->widget.parent)
+  {
+    /* get the recalculated size (we want the width) */
+    GtkRequisition size;
+    gtk_widget_size_request (gtk_widget_get_toplevel (&gtv->widget), &size);
+    gtv->widget.allocation.width = w > size.width ? w : size.width;
+  }
+  else
+    gtv->widget.allocation.width = w;
   gtv->widget.allocation.height = h;
-  gtv->priv->resize = gtv->priv->force_resize = TRUE;
-  XUnlockDisplay (gtv->priv->display);
+  priv->resize = priv->force_resize = TRUE;
+  if (real)
+    XUnlockDisplay (priv->display);
 }
 
 void gtk_video_set_fullscreen (GtkVideo *gtv, gint fullscreen) {
@@ -1115,26 +1171,27 @@ void gtk_video_set_fullscreen (GtkVideo *gtv, gint fullscreen) {
 
   if (fullscreen)
   {
-    XEvent xev;
-
+    setup_fullscreen_window (priv);
     XRaiseWindow(priv->display, priv->fullscreen_window);
-
     XMapWindow (priv->display, priv->fullscreen_window);
-
     XFlush(priv->display);
 
     /* Wait for map. */
+    pthread_mutex_lock (&priv->map_lock);
+    XUnlockDisplay (priv->display);
+    while (!priv->mapped)
+      pthread_cond_wait (&priv->map_cond, &priv->map_lock);
+    pthread_mutex_unlock (&priv->map_lock);
 
-    do  {
-      XWindowEvent(priv->display, priv->fullscreen_window,
-		   StructureNotifyMask, &xev);
-    } while (xev.type != MapNotify || xev.xmap.event != priv->fullscreen_window);
-
+    sched_yield ();
+    XLockDisplay (priv->display);
     XSetInputFocus (priv->display, priv->fullscreen_window, 
 		    RevertToNone, CurrentTime);
 
     XMoveWindow (priv->display, priv->fullscreen_window, 0, 0); 
 
+    XFlush(priv->display);
+
     XUnlockDisplay (priv->display);
     xine_port_send_gui_data (priv->video_port,
 			     XINE_GUI_SEND_WILL_DESTROY_DRAWABLE,
@@ -1208,37 +1265,304 @@ gboolean gtk_video_get_auto_resize (GtkVideo *gtv) {
   return gtv->priv->auto_resize;
 }
 
-static void gtv_hide_vis (gtk_video_private_t *priv,
-			  xine_audio_port_t *audio_port)
+
+/* Plugins: shared post-plugin code */
+
+static GList *gtv_create_plugins (gtk_video_private_t *priv,
+				  xine_audio_port_t **audio,
+				  xine_video_port_t **video,
+				  const char *plugins, int type)
 {
-  if (priv->vis_plugin)
+  GList *list = NULL;
+
+  if (!plugins || !plugins[0])
+    return NULL;
+
+  for (;;)
   {
-    xine_post_wire_audio_port (xine_get_audio_source (priv->stream),
-			       audio_port);
-    xine_post_dispose (priv->xine, priv->vis_plugin);
-    priv->vis_plugin = NULL;
+    char *next = strchr (plugins, ';') ? : (char *)plugins + strlen (plugins);
+    char *args = strchr (plugins, ':');
+
+    if (*plugins != '-')
+    {
+      char *name;
+      xine_post_t *plugin;
+
+      if (args > next)
+	args = NULL;
+
+      logprintf ("plugin: %p %.*s\n", plugins, (args?:next) - plugins, plugins);
+      if (args)
+	logprintf ("args  : %p %.*s\n", args ? args + 1 : NULL,
+		   args ? next - args - 1 : 0, args ? args + 1 : NULL);
+      logprintf ("next  : %p\n", next);
+
+      name = g_strndup (plugins, (args ? : next) - plugins);
+      plugin = xine_post_init (priv->xine, name, 0, audio, video);
+      if (plugin)
+      {
+	if (plugin->type == type)
+	{
+	  list = g_list_append (list, plugin);
+	  if (args)
+	  {
+	    args = g_strndup (args + 1, next - args - 1);
+	    post_parse_set_parameters (plugin, args);
+	    free (args);
+	  }
+	}
+	else
+	{
+	  fprintf (stderr, "gtkvideo: post-plugin '%s': wrong type (wanted %d, got %d)\n", name, type, plugin->type);
+	  xine_post_dispose (priv->xine, plugin);
+	}
+      }
+      else
+	fprintf (stderr, "gtkvideo: unknown post-plugin '%s'\n", name);
+      free (name);
+    }
+
+    if (!*next)
+      break;
+    plugins = next + 1;
+  }
+
+  return list;
+}
+
+static void gtv_dispose_plugin (xine_post_t *post, xine_t *xine)
+{
+  xine_post_dispose (xine, post);
+}
+
+static void gtv_dispose_plugins (gtk_video_private_t *priv, GList *list)
+{
+  g_list_foreach (list, (GFunc) gtv_dispose_plugin, priv->xine);
+  g_list_free (list);
+}
+
+static xine_post_out_t *gtv_post_output (xine_post_t *post,
+					 const char *const s[])
+{
+  return xine_post_output (post, s[0])
+	 ? : xine_post_output (post, s[1])
+	     ? : xine_post_output (post, xine_post_list_outputs (post)[0]);
+}
+
+static xine_post_in_t *gtv_post_input (xine_post_t *post,
+				       const char *const s[])
+{
+  return xine_post_input (post, s[0])
+	 ? : xine_post_input (post, s[1])
+	     ? : xine_post_input (post, xine_post_list_inputs (post)[0]);
+}
+
+/* Plugins: video post-plugins */
+
+static const char *const video_io[] = { "video out", "video", "video in" };
+
+static xine_post_out_t *gtv_post_output_video (xine_post_t *post)
+{
+  return gtv_post_output (post, video_io);
+}
+
+static xine_post_in_t *gtv_post_input_video (xine_post_t *post)
+{
+  return gtv_post_input (post, video_io + 1);
+}
+
+static void gtv_wire_plugins_video (gtk_video_private_t *priv)
+{
+  /* note: assumes default wiring */
+  GList *list = priv->post_video_enable ? g_list_copy (priv->post_video) : NULL;
+  GList *item, *next = NULL;
+
+  if (priv->tvtime_plugin && priv->deinterlace)
+    list = g_list_prepend (list, priv->tvtime_plugin);
+
+  xine_post_wire_video_port (xine_get_video_source (priv->stream),
+			     priv->video_port);
+
+  item = g_list_last (list);
+  while (item)
+  {
+    xine_post_out_t *out = gtv_post_output_video (item->data);
+    if (next)
+      xine_post_wire (out, gtv_post_input_video (next->data));
+    else
+      xine_post_wire_video_port (out, priv->video_port);
+    next = item;
+    item = g_list_previous (item);
+  }
+
+  if (next)
+    xine_post_wire (xine_get_video_source (priv->stream),
+		    gtv_post_input_video (next->data));
+
+  g_list_free (list);
+}
+
+static void gtv_unwire_plugins_video (gtk_video_private_t *priv)
+{
+  GList *item;
+  xine_post_wire_video_port (xine_get_video_source (priv->stream),
+			     priv->video_port);
+  item = g_list_last (priv->post_video);
+  while (item)
+  {
+    xine_post_wire (NULL, gtv_post_input_video (item->data));
+    item = g_list_previous (item);
   }
 }
 
-static gboolean gtv_show_vis (gtk_video_private_t *priv,
+void gtk_video_set_post_plugins_video (GtkVideo *gtv, const char *params)
+{
+  gtk_video_private_t *priv;
+
+  g_return_if_fail (gtv != NULL);
+  g_return_if_fail (GTK_IS_VIDEO (gtv));
+  priv = gtv->priv;
+  g_return_if_fail (priv->stream != NULL);
+
+  gtv_unwire_plugins_video (priv);
+  gtv_dispose_plugins (priv, priv->post_video);
+  priv->post_video = gtv_create_plugins (priv, NULL, &priv->video_port,
+					 params, XINE_POST_TYPE_VIDEO_FILTER);
+  gtv_wire_plugins_video (priv);
+}
+
+void gtk_video_set_use_post_plugins_video (GtkVideo *gtv, gboolean on)
+{
+  gtk_video_private_t *priv;
+
+  g_return_if_fail (gtv != NULL);
+  g_return_if_fail (GTK_IS_VIDEO (gtv));
+  priv = gtv->priv;
+  g_return_if_fail (priv->stream != NULL);
+
+  gtv_unwire_plugins_video (priv);
+  priv->post_video_enable = on;
+  gtv_wire_plugins_video (priv);
+}
+
+gboolean gtk_video_get_use_post_plugins_video (GtkVideo *gtv)
+{
+  return (gtv && gtv->priv) ? gtv->priv->post_video_enable : FALSE;
+}
+
+
+/* Plugins: audio post-plugins */
+
+static const char *const audio_io[] = { "audio out", "audio", "audio in" };
+
+static xine_post_out_t *gtv_post_output_audio (xine_post_t *post)
+{
+  return gtv_post_output (post, audio_io);
+}
+
+static xine_post_in_t *gtv_post_input_audio (xine_post_t *post)
+{
+  return gtv_post_input (post, audio_io + 1);
+}
+
+static void gtv_wire_plugins_audio (gtk_video_private_t *priv,
+				    xine_audio_port_t *audio_port)
+{
+  /* note: assumes default wiring */
+  GList *list = priv->post_audio_enable ? g_list_copy (priv->post_audio) : NULL;
+  GList *item, *next = NULL;
+
+  if (priv->vis_active && priv->vis_plugin)
+    list = g_list_prepend (list, priv->vis_plugin);
+
+  item = g_list_last (list);
+  while (item)
+  {
+    xine_post_out_t *out = gtv_post_output_audio (item->data);
+    if (next)
+      xine_post_wire (out, gtv_post_input_audio (next->data));
+    else
+      xine_post_wire_audio_port (out, audio_port);
+    next = item;
+    item = g_list_previous (item);
+  }
+
+  if (next)
+    xine_post_wire (xine_get_audio_source (priv->stream),
+		    gtv_post_input_audio (next->data));
+
+  g_list_free (list);
+}
+
+static void gtv_unwire_plugins_audio (gtk_video_private_t *priv,
+				      xine_audio_port_t *audio_port)
+{
+  GList *item;
+  xine_post_wire_audio_port (xine_get_audio_source (priv->stream), audio_port);
+  item = g_list_last (priv->post_audio);
+  while (item)
+  {
+    xine_post_wire (NULL, gtv_post_input_audio (item->data));
+    item = g_list_previous (item);
+  }
+  if (priv->vis_plugin)
+    xine_post_wire (NULL, gtv_post_input_audio (priv->vis_plugin));
+}
+
+void gtk_video_set_post_plugins_audio (GtkVideo *gtv, const char *params,
+				       xine_audio_port_t *audio_port)
+{
+  gtk_video_private_t *priv;
+
+  g_return_if_fail (gtv != NULL);
+  g_return_if_fail (GTK_IS_VIDEO (gtv));
+  priv = gtv->priv;
+  g_return_if_fail (priv->stream != NULL);
+
+  gtv_unwire_plugins_audio (priv, audio_port);
+  gtv_dispose_plugins (priv, priv->post_audio);
+  priv->post_audio = gtv_create_plugins (priv, &audio_port, NULL,
+					 params, XINE_POST_TYPE_AUDIO_FILTER);
+  gtv_wire_plugins_audio (priv, audio_port);
+}
+
+void gtk_video_set_use_post_plugins_audio (GtkVideo *gtv, gboolean on,
+					   xine_audio_port_t *audio_port)
+{
+  gtk_video_private_t *priv;
+
+  g_return_if_fail (gtv != NULL);
+  g_return_if_fail (GTK_IS_VIDEO (gtv));
+  priv = gtv->priv;
+  g_return_if_fail (priv->stream != NULL);
+
+  priv->post_audio_enable = on;
+  gtv_unwire_plugins_audio (priv, audio_port);
+  gtv_wire_plugins_audio (priv, audio_port);
+}
+
+gboolean gtk_video_get_use_post_plugins_audio (GtkVideo *gtv)
+{
+  return (gtv && gtv->priv) ? gtv->priv->post_audio_enable : FALSE;
+}
+
+
+/* Plugins: audio visualisation post-plugins */
+
+static void gtv_recreate_vis (gtk_video_private_t *priv,
 			      xine_audio_port_t **audio_port)
 {
-  if (!priv->vis_plugin && priv->vis_plugin_id &&
-      strcmp (priv->vis_plugin_id, "none"))
-  {
+  if (priv->vis_plugin)
+    xine_post_dispose (priv->xine, priv->vis_plugin);
+  if (priv->vis_active && priv->vis_plugin_id)
     priv->vis_plugin = xine_post_init (priv->xine, priv->vis_plugin_id, 0,
 				       audio_port, &priv->video_port);
-    if (!priv->vis_plugin)
-      return FALSE;
-    xine_post_wire (xine_get_audio_source (priv->stream),
-		    xine_post_input (priv->vis_plugin, "audio in"));
-    return TRUE;
-  }
-  return FALSE;
+  else
+    priv->vis_plugin = NULL;
 }
 
-gboolean gtk_video_set_vis (GtkVideo *gtv, const char *id,
-			    xine_audio_port_t **audio_port)
+gboolean gtk_video_select_vis (GtkVideo *gtv, const char *id,
+			       xine_audio_port_t **audio_port)
 {
   gtk_video_private_t *priv;
 
@@ -1247,30 +1571,23 @@ gboolean gtk_video_set_vis (GtkVideo *gtv, const char *id,
   priv = gtv->priv;
   g_return_val_if_fail (priv->stream != NULL, FALSE);
 
+  if (!priv->vis_plugin_id && (!id || !strcasecmp (id, "none")))
+    return FALSE; /* same lack of plugin => nothing to do */
+
+  if (priv->vis_plugin_id && id && !strcasecmp (priv->vis_plugin_id, id))
+    return priv->vis_plugin != NULL; /* same plugin => nothing to do */
+
+  gtv_unwire_plugins_audio (priv, *audio_port);
   free (priv->vis_plugin_id);
-  priv->vis_plugin_id = strdup (id ? id : "none");
+  priv->vis_plugin_id = id ? strdup (id) : NULL;
+  gtv_recreate_vis (priv, audio_port);
+  gtv_wire_plugins_audio (priv, *audio_port);
 
-  if (!priv->vis_active)
-    return priv->vis_plugin != NULL;
-
-  gtv_hide_vis (priv, *audio_port);
-  return gtv_show_vis (priv, audio_port);
+  return priv->vis_plugin != NULL;
 }
 
-void gtk_video_hide_vis (GtkVideo *gtv, xine_audio_port_t *audio_port)
-{
-  gtk_video_private_t  *priv;
-
-  g_return_if_fail (gtv != NULL);
-  g_return_if_fail (GTK_IS_VIDEO (gtv));
-  priv = gtv->priv;
-  g_return_if_fail (priv->stream != NULL);
-
-  priv->vis_active = FALSE;
-  gtv_hide_vis (priv, audio_port);
-}
-
-gboolean gtk_video_show_vis (GtkVideo *gtv, xine_audio_port_t **audio_port)
+gboolean gtk_video_set_vis (GtkVideo *gtv, xine_audio_port_t **audio_port,
+			    gboolean active)
 {
   gtk_video_private_t  *priv;
 
@@ -1279,9 +1596,92 @@ gboolean gtk_video_show_vis (GtkVideo *gtv, xine_audio_port_t **audio_port)
   priv = gtv->priv;
   g_return_val_if_fail (priv->stream != NULL, FALSE);
 
-  if (priv->vis_active)
-    return priv->vis_plugin != NULL;
+  if (priv->vis_active == active)
+    return priv->vis_plugin != NULL; /* same state => nothing to do */
 
-  priv->vis_active = TRUE;
-  return gtv_show_vis (priv, audio_port);
+  priv->vis_active = active;
+  gtv_unwire_plugins_audio (priv, *audio_port);
+  gtv_recreate_vis (priv, audio_port);
+  gtv_wire_plugins_audio (priv, *audio_port);
+
+  return priv->vis_plugin != NULL;
+}
+
+
+/* Plugins: deinterlace post-plugin */
+
+void gtk_video_set_tvtime (GtkVideo *gtv, const char *params)
+{
+  gtk_video_private_t *priv;
+  char *d, *v;
+
+  g_return_if_fail (gtv != NULL);
+  g_return_if_fail (GTK_IS_VIDEO (gtv));
+  priv = gtv->priv;
+  g_return_if_fail (priv->stream != NULL);
+
+  if (!priv->tvtime_plugin || strncmp (params, "tvtime:", 7))
+    return;
+
+  d = strchr (params, ':');
+  v = strdup (v ? d + 1 : params);
+  d = strchr (v, ';');
+  if (d)
+    *d = 0;
+  post_parse_set_parameters (priv->tvtime_plugin, v);
+  free (v);
+}
+
+void gtk_video_set_deinterlace (GtkVideo *gtv, gboolean set)
+{
+  gtk_video_private_t *priv;
+
+  g_return_if_fail (gtv != NULL);
+  g_return_if_fail (GTK_IS_VIDEO (gtv));
+  priv = gtv->priv;
+  g_return_if_fail (priv->stream != NULL);
+
+  priv->deinterlace = set;
+  if (priv->tvtime_plugin)
+  {
+    gtv_unwire_plugins_video (priv);
+    gtv_wire_plugins_video (priv);
+  }
+  else
+    xine_set_param (priv->stream, XINE_PARAM_VO_DEINTERLACE, set);
+}
+
+gboolean gtk_video_get_deinterlace (GtkVideo *gtv)
+{
+  return (gtv && gtv->priv) ? gtv->priv->deinterlace : FALSE;
+}
+
+xine_post_t *gtk_video_get_deinterlacer (GtkVideo *gtv)
+{
+  return (gtv && gtv->priv) ? gtv->priv->tvtime_plugin : NULL;
+}
+
+void gtk_video_set_auto_rescale (GtkVideo *gtv, gboolean set)
+{
+  gtk_video_private_t *priv;
+
+  g_return_if_fail (gtv != NULL);
+  g_return_if_fail (GTK_IS_VIDEO (gtv));
+  priv = gtv->priv;
+  g_return_if_fail (priv->stream != NULL);
+
+  priv->auto_rescale = set;
+
+  pthread_mutex_lock (&resize_lock);
+  if (!priv->resizing)
+  {
+    priv->resizing = TRUE;
+    g_idle_add ((GSourceFunc) gtk_video_idle_resize, gtv);
+  }
+  pthread_mutex_unlock (&resize_lock);
+}
+
+gboolean gtk_video_get_auto_rescale (GtkVideo *gtv)
+{
+  return (gtv && gtv->priv) ? gtv->priv->auto_rescale : FALSE;
 }
