@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002 the xine project
+ * Copyright (C) 2002-2005 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: http.c,v 1.13 2004/12/17 00:32:59 dsalt Exp $
+ * $Id: http.c,v 1.17 2005/01/16 17:38:34 dsalt Exp $
  */
 
 #include "globals.h"
@@ -282,7 +282,7 @@ static void report_error (const char *url, const char *msg) {
 static http_t *http_open (const char *mrl) {
 
   http_t   *this;
-  char     *proxy;
+  char     *proxy, *request = NULL;
   int       done,len,linenum;
   char      mime_type[BUFSIZE];
 
@@ -354,44 +354,44 @@ static http_t *http_open (const char *mrl) {
   }
 
   if (proxy != NULL)
+  {
     if (this->port != DEFAULT_HTTP_PORT)
-      sprintf (this->buf, "GET http://%s:%d/%s HTTP/1.0\015\012",
-	       this->host, this->port, this->filename);
+      request = g_strdup_printf ("GET http://%s:%d/%s HTTP/1.0\015\012",
+		       this->host, this->port, this->filename);
     else
-      sprintf (this->buf, "GET http://%s/%s HTTP/1.0\015\012",
-	       this->host, this->filename);
+      request = g_strdup_printf ("GET http://%s/%s HTTP/1.0\015\012",
+		       this->host, this->filename);
+  }
   else
-    sprintf (this->buf, "GET /%s HTTP/1.0\015\012", this->filename);
-  
+    request = g_strdup_printf ("GET /%s HTTP/1.0\015\012", this->filename);
+
   if (this->port != DEFAULT_HTTP_PORT)
-    sprintf (this->buf + strlen(this->buf), "Host: %s:%d\015\012",
-	     this->host, this->port);
+    asreprintf (&request, "%sHost: %s:%d\015\012", request,
+		this->host, this->port);
   else
-    sprintf (this->buf + strlen(this->buf), "Host: %s\015\012",
-	     this->host);
-  
+    asreprintf (&request, "%sHost: %s\015\012", request, this->host);
+
   if (this->proxyuser != NULL)
-    sprintf (this->buf + strlen(this->buf), "Proxy-Authorization: Basic %s\015\012",
-	     this->proxyauth);
-  
+    asreprintf (&request, "%sProxy-Authorization: Basic %s\015\012", request,
+		this->proxyauth);
+
   if (this->user != NULL)
-    sprintf (this->buf + strlen(this->buf), "Authorization: Basic %s\015\012",
+    asreprintf (&request, "%sAuthorization: Basic %s\015\012", request,
 	     this->auth);
-  
-  sprintf (this->buf + strlen(this->buf), "User-Agent: xine/%s\015\012",
-	   VERSION);
-  
-  strcat (this->buf, "Accept: */*\015\012");
-  
-  strcat (this->buf, "\015\012");
-  
-  if (write (this->fh, this->buf, strlen(this->buf)) != strlen(this->buf)) {
+
+  asreprintf (&request,
+	      "%sUser-Agent: xine/%s\015\012Accept: */*\015\012\015\012",
+	      request, VERSION);
+
+  if (write (this->fh, request, strlen(request)) != strlen(request)) {
+    free (request);
     free (this);
     report_error (mrl, _("couldn't send request"));
-    return NULL ;
+    return NULL;
   }
 
-  logprintf ("http: request sent: >%s<\n", this->buf);
+  logprintf ("http: request sent: >%s<\n", request);
+  free (request);
 
   /* read and parse reply */
   done = 0; len = 0; linenum = 0;
