@@ -16,7 +16,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
  *
- * $Id: infobar.c,v 1.18 2004/12/03 22:01:22 dsalt Exp $
+ * $Id: infobar.c,v 1.22 2005/01/12 18:44:14 dsalt Exp $
  *
  * a nice black information display area
  */
@@ -31,7 +31,7 @@
 
 #include "infobar.h"
 #include "utils.h"
-#include "globals.h"
+#include "playlist.h"
 #include "player.h"
 
 #define NUMLINES 3
@@ -150,19 +150,23 @@ void infobar_show_metadata (void)
       infobar_line (bar, 0, "%s", t);
     }
   } else {
-    if (strlen (player_get_cur_mrl()) > 0) {
-
-      char *str = strrchr (player_get_cur_mrl(), '/');
-
-      if (str) {
-	strbuf = g_strdup_printf ("%s - gxine %s", str+1, VERSION);
-	infobar_line (bar, 0, "%s", str+1);
-      } else {
-        str = player_get_cur_mrl ();
-	strbuf = g_strdup_printf ("%s - gxine %s", str, VERSION);
-	infobar_line (bar, 0, "%s", str);
+    const char *str = NULL;
+    if (!playlist_showing_logo ())
+    {
+      str = player_get_cur_title ();
+      if (!str || !str[0])
+      {
+        const char *mrl = player_get_cur_mrl ();
+        str = (strrchr (mrl, '/') ? : mrl - 1) + 1; /* leaf */
       }
-    } else {
+    }
+    if (str && str[0])
+    {
+      strbuf = g_strdup_printf ("%s - gxine %s", str, VERSION);
+      infobar_line (bar, 0, "%s", str);
+    }
+    else
+    {
       strbuf = g_strdup_printf ("gxine %s", VERSION);
       infobar_line (bar, 0, "%s", strbuf);
     }
@@ -225,13 +229,16 @@ infobar_t *create_infobar (GtkWidget **w) {
   return bar;
 }
 
-void postinit_infobar (GtkWidget *window, infobar_t *bar)
+void postinit_infobar (infobar_t *bar)
 {
   int i;
+  GtkWidget *window;
 
   if (!bar) return;
 
-  g_signal_connect (bar->w, "expose_event", 
+  window = gtk_widget_get_toplevel (bar->w);
+
+  g_signal_connect (bar->w, "expose-event", 
 		    G_CALLBACK (expose_event_cb), bar);
 
   bar->black_gc = gdk_gc_new (window->window);
@@ -247,18 +254,11 @@ void postinit_infobar (GtkWidget *window, infobar_t *bar)
    * pango stuff (antialiased font rendering
    */
 
-  bar->pfd = pango_font_description_new ();
-  pango_font_description_set_family (bar->pfd, "Sans");
-  pango_font_description_set_weight (bar->pfd, PANGO_WEIGHT_BOLD);
-  pango_font_description_set_size (bar->pfd, 8*PANGO_SCALE);
-  bar->pfd_big = pango_font_description_new ();
-  pango_font_description_set_family (bar->pfd_big, "Sans");
-  pango_font_description_set_weight (bar->pfd_big, PANGO_WEIGHT_BOLD);
-  pango_font_description_set_size (bar->pfd_big, 12*PANGO_SCALE);
-
+  bar->pfd_big = pango_font_description_from_string ("Sans Bold 12");
   bar->pl[0] = gtk_widget_create_pango_layout (bar->w, NULL);
   pango_layout_set_font_description (bar->pl[0], bar->pfd_big);
 
+  bar->pfd = pango_font_description_from_string ("Sans Bold 8");
   for (i = 1; i < NUMLINES; ++i)
   {
     bar->pl[i] = gtk_widget_create_pango_layout (bar->w, NULL);
