@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2004 the xine-project
+ * Copyright (C) 2002-2005 the xine-project
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -16,7 +16,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
  *
- * $Id: infobar.h,v 1.3 2004/11/26 22:57:01 dsalt Exp $
+ * $Id: infobar.h,v 1.4 2005/01/12 18:44:14 dsalt Exp $
  *
  * a nice black information display area
  */
@@ -29,7 +29,7 @@ typedef struct infobar_s infobar_t;
 
 infobar_t *create_infobar (GtkWidget **w);
 
-void postinit_infobar (GtkWidget *window, infobar_t *bar) ;
+void postinit_infobar (infobar_t *bar);
 
 void infobar_show_metadata (void);
 
