@@ -16,7 +16,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
  *
- * $Id: mediamarks.c,v 1.37 2004/12/17 00:33:00 dsalt Exp $
+ * $Id: mediamarks.c,v 1.42.2.1 2005/07/12 18:44:33 dsalt Exp $
  *
  * mediamarks - load
  *            - save
@@ -450,12 +450,17 @@ static int load_mediamarks (gchar *fname)
     xml_node_t *node;
 
     xml_parser_init (mmfile, strlen (mmfile), XML_PARSER_CASE_INSENSITIVE);
+    g_signal_handlers_block_by_func (G_OBJECT(mm_store), tree_changed_cb, NULL);
 
     if (xml_parser_build_tree (&node)<0) {
       load_old_file (fname);
     } else 
       load_new_file (node);
 
+    update_menu (media_menu[0]);
+    update_menu (media_menu[1]);
+    g_signal_handlers_unblock_by_func (G_OBJECT(mm_store), tree_changed_cb, NULL);
+
     free (mmfile);
   } else
     return 0;
@@ -478,15 +483,17 @@ void mm_import (void) {
   }
 }
 
-void mm_add_show () {
-
+void mm_add_show (void)
+{
   play_item_t *play_item;
   GtkTreeIter  iter;
   int          pos, pos_time, len;
   char	      *dot;
   const char  *title = xine_get_meta_info (stream, XINE_META_INFO_TITLE);
 
-  play_item = play_item_copy (playlist_get_item (playlist_get_list_pos()));
+  play_item = playlist_get_item (playlist_get_list_pos());
+  play_item = play_item ? play_item_copy (play_item)
+			: play_item_new ("", "", 0);
 
   /* default title? try to get title from meta-info */
   dot = strrchr (play_item->mrl, '.');
@@ -685,11 +692,14 @@ void mediamarks_init (void) {
 
   if (!load_mediamarks (fname)) {
     /* load sample mediamarks */
-
     g_free (fname);
-
     fname = g_strconcat (miscdir, "/mediamarks", NULL);
-    load_mediamarks (fname);
+    //load_mediamarks (fname);
+    if (!load_mediamarks (fname))
+    {
+      asreprintf (&fname, "%s.gz", fname);
+      load_mediamarks (fname);
+    }
   }
 
   g_free(fname);
@@ -727,16 +737,28 @@ void mediamarks_init (void) {
   gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tree), TRUE);
 
   cell = gtk_cell_renderer_text_new ();
+  if (GTK_TEST_COMPAT_2_6)
+    g_object_set (G_OBJECT(cell),
+		  "ellipsize", 2, "width_chars", 16, NULL); /* foo...bar */
   column = gtk_tree_view_column_new_with_attributes (_("Media mark"), cell,
 						     "text", 0, NULL);
+  gtk_tree_view_column_set_resizable (GTK_TREE_VIEW_COLUMN(column), TRUE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (tree),
 			       GTK_TREE_VIEW_COLUMN (column));
 
-  column = gtk_tree_view_column_new_with_attributes ("MRL", cell,
+  if (GTK_TEST_COMPAT_2_6)
+  {
+    cell = gtk_cell_renderer_text_new ();
+    g_object_set (G_OBJECT(cell),
+		  "ellipsize", 1, "width_chars", 16, NULL); /* ...bar */
+  }
+  column = gtk_tree_view_column_new_with_attributes (_("MRL"), cell,
 						     "text", 1, NULL);
+  gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN(column),
+				   GTK_TREE_VIEW_COLUMN_AUTOSIZE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (tree),
 			       GTK_TREE_VIEW_COLUMN (column));
-  
+
   gtk_tree_view_set_reorderable (GTK_TREE_VIEW (tree), TRUE);
 
   scrolled_window = gtk_scrolled_window_new (NULL, NULL);
@@ -761,7 +783,8 @@ void mediamarks_init (void) {
   g_signal_connect (GTK_OBJECT(b), "clicked",
 		    G_CALLBACK(new_category_cb), tree);
 
-  b = gtk_button_new_with_mnemonic (_("_Edit"));
+  b = GTK_COMPAT_2_6 (gtk_button_new_from_stock ("gtk-edit"),
+		      gtk_button_new_with_mnemonic (_("_Edit")));
   gtk_box_pack_start (GTK_BOX(hbox), b, FALSE, FALSE, 0);
   g_signal_connect (GTK_OBJECT(b), "clicked",
 		    G_CALLBACK(edit_cb), NULL);
