@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2003 the xine project
+ * Copyright (C) 2003-2005 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: menu.c,v 1.33 2004/12/15 19:23:19 dsalt Exp $
+ * $Id: menu.c,v 1.41.2.1 2005/04/29 20:01:44 dsalt Exp $
  *
  * main/context menu creation / handling
  */
@@ -23,6 +23,7 @@
 #include "globals.h"
 
 #include <string.h>
+#include <stdlib.h>
 #include <gdk/gdkkeysyms.h>
 
 #include "menu.h"
@@ -36,10 +37,14 @@
 
 GtkWidget *popup_menu = NULL, *menubar = NULL;
 
-GtkMenuShell *media_menu[2] = { NULL, NULL };
+GtkMenuShell *media_menu[2] = { NULL, NULL },
+	     *vis_menu[2] = { NULL, NULL };
 GtkCheckMenuItem *fullscreen_menu_item[2] = { NULL, NULL },
 		 *deinterlace_menu_item[2] = { NULL, NULL },
+		 *pp_video_menu_item[2] = { NULL, NULL },
+		 *pp_audio_menu_item[2] = { NULL, NULL },
 		 *auto_resize_menu_item[2] = { NULL, NULL },
+		 *auto_rescale_menu_item[2] = { NULL, NULL },
 		 *toolbar_item[3][2] = { { NULL, NULL }, { NULL, NULL },
 					 { NULL, NULL } };
 GSList *aspect_menu_items[2] = { NULL, NULL };
@@ -145,97 +150,131 @@ static void add_autoplay_entries (GtkMenuShell *submenu, int index) {
   }
 }
 
-static void vis_cb (GtkWidget* widget, gpointer data) {
-
-  /* printf ("menu: vis_cb %s\n", data); */
-
-  if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget))) {
-    xine_cfg_entry_t vis_entry;
-
-    if (xine_config_lookup_entry (xine, "misc.vis", &vis_entry))
-    {
-      vis_entry.str_value = data;
-      xine_config_update_entry (xine, &vis_entry);
-    }
-
-    pthread_mutex_lock (&engine_lock);
-    vis_set (GTK_VIDEO(gtv), data, &audio_port);
-    pthread_mutex_unlock (&engine_lock);
-  }
-}
-
-static void add_vis_entries (GtkMenuShell *submenu, 
-			     GtkRadioMenuItem *none_item) {
-
-  GtkWidget         *item;
-  int                i;
-  const char        *vis;
-  GSList            *group;
-  const char *const *pol;
-
-  vis = xine_config_register_string (xine, "misc.vis", "none",
-				     _("Default visualisation setting"),
-				     NULL, 40, NULL, CONFIG_DATA_NONE);
-
-  group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (none_item));
-  if (!strcasecmp (vis, "none"))
-    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(none_item), TRUE);
-
-  pol = xine_list_post_plugins_typed (xine, XINE_POST_TYPE_AUDIO_VISUALIZATION);
-  if (pol) {
-
-    i = 0;
-    while (pol[i]) {
-
-      /* item = gtk_menu_item_new_with_label (pol[i]); */
-      item = gtk_radio_menu_item_new_with_label (group, pol[i]);
-      group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM(item));
-
-      g_signal_connect (GTK_OBJECT(item), "activate", 
-			G_CALLBACK (vis_cb), (gpointer) strdup (pol[i]));
-
-      gtk_menu_shell_insert (submenu, item, 1);
-      gtk_widget_show(item);
-
-      if (!strcasecmp (vis, pol[i]))
-	gtk_check_menu_item_set_active ( GTK_CHECK_MENU_ITEM(item), TRUE);
-
-      i++;
-    }
-  }
-}
-
-static void open_cb (void) {
-  engine_exec ("open_show ();", NULL, NULL);
-}
-
-static void open_mrl_cb(void) {
-  engine_exec ("open_mrl_show ();", NULL, NULL);
-}
-
-static void playlist_cb(void) {
-  engine_exec ("playlist_show ();", NULL, NULL);
-}
-
-static void preferences_cb(void) {
-  engine_exec ("preferences_show ();", NULL, NULL);
-}
-
-static void keybindings_cb(void) {
-  engine_exec ("keybindings_show ();", NULL, NULL);
-}
-
-static void exit_cb(void) {
-  engine_exec ("exit ();", NULL, NULL);
-}
-
-static void fullscreen_cb(void) {
-  engine_exec ("set_fullscreen ();", NULL, NULL);
-}
-
-static void tbar_toggle_cb (void)
+static void vis_none_cb (gpointer callback_data, guint callback_action,
+			 GtkWidget *widget)
 {
-  engine_exec ("toolbar_show ();", NULL, NULL);
+  vis_set (GTK_VIDEO(gtv), "none", &audio_port);
+}
+
+static void vis_cb (GtkWidget* widget, gpointer data)
+{
+  vis_set (GTK_VIDEO(gtv), data, &audio_port);
+}
+
+static void add_vis_entries (GtkMenuShell *submenu, GtkRadioMenuItem *none_item)
+{
+  const char *const *pol =
+    xine_list_post_plugins_typed (xine, XINE_POST_TYPE_AUDIO_VISUALIZATION);
+  if (pol)
+  {
+    xine_cfg_entry_t entry;
+    gboolean ticked = FALSE;
+    int i;
+    GSList *group =
+      gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (none_item));
+    const char *vis = xine_config_lookup_entry
+			(xine, "gui.post_plugins.audio_visualisation", &entry)
+		      ? entry.enum_values[entry.num_value] : NULL;
+
+    for (i = 0; pol[i]; ++i)
+    {
+      GtkWidget *item = gtk_radio_menu_item_new_with_label (group, pol[i]);
+//      group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM(item));
+      g_object_set_data (G_OBJECT (item), "vis", (gpointer) strdup (pol[i]));
+      g_signal_connect (GTK_OBJECT(item), "activate",
+			G_CALLBACK (vis_cb), (gpointer) strdup (pol[i]));
+      gtk_menu_shell_insert (submenu, item, i + 1);
+      gtk_widget_show(item);
+      if (vis && !strcasecmp (vis, pol[i]))
+	GTK_CHECK_MENU_ITEM(item)->active = ticked = TRUE;
+    }
+    GTK_CHECK_MENU_ITEM(none_item)->active = !ticked;
+  }
+}
+
+enum {
+  /* File */
+  JS_CB_OPEN,
+  JS_CB_OPEN_MRL,
+  JS_CB_PLAYLIST,
+  JS_CB_PREFERENCES,
+  JS_CB_KEYBINDINGS,
+  JS_CB_SNAPSHOT,
+  JS_CB_EXIT,
+  /* View */
+  JS_CB_FULLSCREEN,
+  JS_CB_TOOLBAR,
+  JS_CB_ZOOM_IN,
+  JS_CB_ZOOM_OUT,
+  JS_CB_ZOOM_100,
+  JS_CB_AV_SETTINGS,
+  JS_CB_POSTPROC_DEINTERLACE,
+  JS_CB_POSTPROC_VIDEO,
+  JS_CB_POSTPROC_AUDIO,
+  /* Media */
+  JS_CB_MEDIAMARKS_ADD,
+  JS_CB_MEDIAMARKS_MANAGE,
+  JS_CB_MEDIAMARKS_IMPORT,
+  /* Help */
+  JS_CB_ABOUT,
+  JS_CB_LOG,
+  JS_CB_STREAM_INFO,
+  JS_CB_WIZARDS,
+};
+
+static void js_cb (gpointer data, guint action, GtkWidget *widget)
+{
+  static const char *const cmds[] = {
+    /* File */
+    [JS_CB_OPEN]			= "open_show ();",
+    [JS_CB_OPEN_MRL]			= "open_mrl_show ();",
+    [JS_CB_PLAYLIST]			= "playlist_show ();",
+    [JS_CB_PREFERENCES]			= "preferences_show ();",
+    [JS_CB_KEYBINDINGS]			= "keybindings_show ();",
+    [JS_CB_SNAPSHOT]			= "snapshot ();",
+    [JS_CB_EXIT]			= "exit ();",
+    /* View */
+    [JS_CB_FULLSCREEN]			= "set_fullscreen ();",
+    [JS_CB_TOOLBAR]			= "toolbar_show ();",
+    [JS_CB_ZOOM_IN]			= "set_zoom (get_zoom()+5);",
+    [JS_CB_ZOOM_OUT]			= "set_zoom (get_zoom()-5);",
+    [JS_CB_ZOOM_100]			= "set_zoom (100);",
+    [JS_CB_AV_SETTINGS]			= "settings_show ();",
+    [JS_CB_POSTPROC_DEINTERLACE]	= "deinterlace_show ();",
+    [JS_CB_POSTPROC_VIDEO]		= "postproc_video_show ();",
+    [JS_CB_POSTPROC_AUDIO]		= "postproc_audio_show ();",
+    /* Media */
+    [JS_CB_MEDIAMARKS_ADD]		= "mm_add_show ();",
+    [JS_CB_MEDIAMARKS_MANAGE]		= "mm_manage_show ();",
+    [JS_CB_MEDIAMARKS_IMPORT]		= "import_mediamarks ();",
+    /* Help */
+    [JS_CB_ABOUT]			= "about_show ();",
+    [JS_CB_LOG]				= "log_show ();",
+    [JS_CB_STREAM_INFO]			= "stream_info_show ();",
+    [JS_CB_WIZARDS]			= "run_wizards ();",
+  };
+  engine_exec (cmds[action], NULL, NULL);
+}
+
+enum {
+  JS_TOGGLE_CB_DEINTERLACE,
+  JS_TOGGLE_CB_AUTO_RESIZE,
+  JS_TOGGLE_CB_AUTO_RESCALE,
+  JS_TOGGLE_CB_POSTPROC_VIDEO,
+  JS_TOGGLE_CB_POSTPROC_AUDIO,
+};
+
+static void js_toggle_cb (gpointer data, guint action, GtkWidget *widget)
+{
+  static const char *const cmds[] = {
+    [JS_TOGGLE_CB_DEINTERLACE]		= "set_deinterlace",
+    [JS_TOGGLE_CB_AUTO_RESIZE]		= "set_auto_resize",
+    [JS_TOGGLE_CB_AUTO_RESCALE]		= "set_auto_rescale",
+    [JS_TOGGLE_CB_POSTPROC_VIDEO]	= "set_postproc_video",
+    [JS_TOGGLE_CB_POSTPROC_AUDIO]	= "set_postproc_audio",
+  };
+  v_engine_exec ("%s (%d);", NULL, NULL, cmds[action],
+		 gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget)));
 }
 
 static void tbar_position_cb(gpointer data, guint action, GtkWidget *widget)
@@ -248,92 +287,11 @@ static void factor_cb(gpointer data, guint action, GtkWidget *widget)
   v_engine_exec ("set_video_size (%d);", NULL, NULL, action);
 }
 
-static void zoom_in_cb(void) {
-  engine_exec ("set_zoom (get_zoom()+5);", NULL, NULL);
-}
-static void zoom_out_cb(void) {
-  engine_exec ("set_zoom (get_zoom()-5);", NULL, NULL);
-}
-static void zoom_100_cb(void) {
-  engine_exec ("set_zoom (100);", NULL, NULL);
-}
-
 static void aspect_cb(gpointer data, guint action, GtkWidget *widget)
 {
   v_engine_exec ("set_aspect (%d);", NULL, NULL, action);
 }
 
-static void deinterlace_cb (gpointer callback_data,
-			    guint callback_action,
-			    GtkWidget *widget) {
-
-  if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget)))
-    engine_exec ("set_deinterlace (1);", NULL, NULL);
-  else
-    engine_exec ("set_deinterlace (0);", NULL, NULL);
-}
-
-static void auto_resize_cb (gpointer callback_data,
-			    guint callback_action,
-			    GtkWidget *widget) {
-
-  if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget)))
-    engine_exec ("set_auto_resize (1);", NULL, NULL);
-  else
-    engine_exec ("set_auto_resize (0);", NULL, NULL);
-}
-
-static void mediamarks_add_cb(void) {
-  engine_exec ("mm_add_show ();", NULL, NULL);
-}
-
-static void mediamarks_manage_cb(void) {
-  engine_exec ("mm_manage_show ();", NULL, NULL);
-}
-
-static void mediamarks_import_cb(void) {
-  engine_exec ("import_mediamarks ();", NULL, NULL);
-}
-
-static void about_cb(void) {
-  engine_exec ("about_show ();", NULL, NULL);
-}
-
-static void log_cb(void) {
-  engine_exec ("log_show ();", NULL, NULL);
-}
-
-static void stream_info_cb(void) {
-  engine_exec ("stream_info_show ();", NULL, NULL);
-}
-
-static void wizards_cb(void) {
-  engine_exec ("run_wizards ();", NULL, NULL);
-}
-
-static void snapshot_cb(void) {
-  engine_exec ("snapshot ();", NULL, NULL);
-}
-
-static void vis_none_cb (gpointer callback_data,
-			 guint callback_action,
-			 GtkWidget *widget) {
-  if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget))) {
-    xine_cfg_entry_t vis_entry;
-
-    if (xine_config_lookup_entry (xine, "misc.vis", &vis_entry))
-    {
-      vis_entry.str_value = "none";
-      xine_config_update_entry (xine, &vis_entry);
-    }
-    engine_exec ("set_vis (\"none\");", NULL, NULL);
-  }
-}
-
-static void settings_cb(void) {
-  engine_exec ("settings_show ();", NULL, NULL);
-}
-
 static void sub_cb (gpointer data, guint action, GtkWidget *widget)
 {
   if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget)))
@@ -357,44 +315,54 @@ static gboolean popup_key_cb (GtkWidget *widget, GdkEventKey *event,
 }
 
 #define STOCK(x) "<StockItem>", GTK_STOCK_##x
+#define STOCKQ(x) "<StockItem>", "gtk-" #x
 #define ASPECT(x) aspect_cb, XINE_VO_ASPECT_##x
+#define JS(x) js_cb, JS_CB_##x
+#define JSTOGGLE(x) js_toggle_cb, JS_TOGGLE_CB_##x
 static GtkItemFactoryEntry menu_items[] = {
   { N_("/_File"),			NULL,		NULL, 0,		"<Branch>" },
-  { N_("/File/_Open..."),		"<control>O",	open_cb, 0,		STOCK(OPEN) },
-  { N_("/File/Open _MRL..."),		"<control>M",	open_mrl_cb, 0,		NULL },
-  { N_("/File/Play_list..."),		NULL,     	playlist_cb, 0,		NULL },
+  { N_("/File/_Open..."),		"<control>O",	JS(OPEN),		STOCK(OPEN) },
+  { N_("/File/Open _MRL..."),		"<control>M",	JS(OPEN_MRL),		STOCK(OPEN) },
+  { N_("/File/Play_list..."),		NULL,     	JS(PLAYLIST),		NULL },
   { "/File/sep0",			NULL,		NULL, 0,		"<Separator>" },
-  { N_("/File/_Preferences..."),	NULL,		preferences_cb, 0,	STOCK(PREFERENCES) },
-  { N_("/File/_Keybindings..."),	NULL,		keybindings_cb, 0,	NULL },
+  { N_("/File/_Preferences..."),	NULL,		JS(PREFERENCES),	STOCK(PREFERENCES) },
+  { N_("/File/_Keybindings..."),	NULL,		JS(KEYBINDINGS),	STOCK(PREFERENCES) },
   { "/File/sep1",			NULL,		NULL, 0,		"<Separator>" },
   { "/File/sep2",			NULL,		NULL, 0,		"<Separator>" },
-  { N_("/File/_Snapshot..."),		"<control>S",	snapshot_cb, 0,		NULL },
+  { N_("/File/_Snapshot..."),		"<control>S",	JS(SNAPSHOT),		NULL },
   { "/File/sep3",			NULL,		NULL, 0,		"<Separator>" },
-  { N_("/File/_Quit"),			"<control>Q",	exit_cb, 0,		STOCK(QUIT) },
+  { N_("/File/_Quit"),			"<control>Q",	JS(EXIT),		STOCK(QUIT) },
   { N_("/_View"),			NULL,		NULL, 0,		"<Branch>" },
-  { N_("/View/_Fullscreen mode"),	"<control>F",	fullscreen_cb, 0,	"<CheckItem>" },
+  { N_("/View/_Fullscreen mode"),	"<control>F",	JS(FULLSCREEN),		"<CheckItem>" },
   { N_("/View/Fullscreen _toolbar"),	NULL,		NULL, 0,		"<Branch>" },
-  { N_("/View/Fullscreen toolbar/_Visible"),	NULL,	tbar_toggle_cb, 0,	"<CheckItem>" },
+  { N_("/View/Fullscreen toolbar/_Visible"),	NULL,	JS(TOOLBAR),		"<CheckItem>" },
   { N_("/View/Fullscreen toolbar/At _top"),	NULL,	tbar_position_cb, 1,	"<RadioItem>" },
   { N_("/View/Fullscreen toolbar/At _bottom"),	NULL,	tbar_position_cb, 0,	"/View/Fullscreen toolbar/At top" },
   { "/View/sep3",			NULL,		NULL, 0,		"<Separator>" },
-  { N_("/View/_50%"),			NULL,		factor_cb, 50,		NULL },
-  { N_("/View/_100%"),			NULL,		factor_cb, 100,		NULL },
-  { N_("/View/_200%"),			NULL,		factor_cb, 200,		NULL },
-  { "/View/sep4",			NULL,		NULL, 0,		"<Separator>" },
-  { N_("/View/Zoom _in"),		NULL,		zoom_in_cb, 0,		STOCK(ZOOM_IN) },
-  { N_("/View/Zoom _out"),		NULL,		zoom_out_cb, 0,		STOCK(ZOOM_OUT) },
-  { N_("/View/_Zoom 100%"),		NULL,		zoom_100_cb, 0,		STOCK(ZOOM_100) },
+  { N_("/View/_Window"),		NULL,		NULL, 0,		"<Branch>" },
+  { N_("/View/Window/_50%"),		NULL,		factor_cb, 50,		NULL },
+  { N_("/View/Window/_100%"),		NULL,		factor_cb, 100,		STOCK(ZOOM_100) },
+  { N_("/View/Window/_200%"),		NULL,		factor_cb, 200,		NULL },
+  { N_("/View/Window/Auto _resize"),	NULL,		JSTOGGLE(AUTO_RESIZE),	"<CheckItem>" },
+  { N_("/View/Window/_Magnify low-res video"),	NULL,	JSTOGGLE(AUTO_RESCALE),	"<CheckItem>" },
+  { N_("/View/Zoom _in"),		NULL,		JS(ZOOM_IN),		STOCK(ZOOM_IN) },
+  { N_("/View/Zoom _out"),		NULL,		JS(ZOOM_OUT),		STOCK(ZOOM_OUT) },
+  { N_("/View/_Zoom 100%"),		NULL,		JS(ZOOM_100),		STOCK(ZOOM_100) },
   { "/View/sep5",			NULL,		NULL, 0,		"<Separator>" },
-  { N_("/View/_Deinterlace"),		"<control>I",	deinterlace_cb, 0,	"<CheckItem>" },
-  { N_("/View/Auto _resize"),		NULL,		auto_resize_cb, 0,	"<CheckItem>" },
+  { N_("/View/_Deinterlace"),		"<control>I",	JSTOGGLE(DEINTERLACE),	"<CheckItem>" },
   { N_("/View/_Aspect ratio"),		NULL,		NULL, 0,		"<Branch>" },
   { N_("/View/Aspect ratio/_Auto"),	NULL,		ASPECT(AUTO),		"<RadioItem>" },
   { N_("/View/Aspect ratio/_Square"),	NULL,		ASPECT(SQUARE),		"/View/Aspect ratio/Auto"},
   { N_("/View/Aspect ratio/_4:3"),	NULL,		ASPECT(4_3),		"/View/Aspect ratio/Auto"},
   { N_("/View/Aspect ratio/_16:9"),	NULL,		ASPECT(ANAMORPHIC),	"/View/Aspect ratio/Auto"},
   { N_("/View/Aspect ratio/_2:1"),	NULL,		ASPECT(DVB),		"/View/Aspect ratio/Auto"},
-  { N_("/View/A\\/V _settings..."),	NULL,		settings_cb, 0,		NULL },
+  { N_("/View/A\\/V _settings..."),	NULL,		JS(AV_SETTINGS),	NULL },
+  { N_("/View/_Post-processing"),	NULL,		NULL, 0,		"<Branch>" },
+  { N_("/View/Post-processing/_Deinterlace..."), NULL,	JS(POSTPROC_DEINTERLACE), STOCK(PREFERENCES) },
+  { N_("/View/Post-processing/_Video..."), NULL,	JS(POSTPROC_VIDEO),	STOCK(PREFERENCES) },
+  { N_("/View/Post-processing/_Audio..."), NULL,	JS(POSTPROC_AUDIO),	STOCK(PREFERENCES) },
+  { N_("/View/Post-processing/Enable _video"), NULL,	JSTOGGLE(POSTPROC_VIDEO), "<CheckItem>" },
+  { N_("/View/Post-processing/Enable _audio"), NULL,	JSTOGGLE(POSTPROC_AUDIO), "<CheckItem>" },
   { "/View/sep6",			NULL,		NULL, 0,		"<Separator>" },
   { N_("/View/_Visualisations"),	NULL,		NULL, 0,		"<Branch>" },
   { N_("/View/Visualisations/_None"),	NULL,		vis_none_cb, 0,		"<RadioItem>"},
@@ -411,16 +379,16 @@ static GtkItemFactoryEntry menu_items[] = {
   { N_("/View/Subtitles/Channel _6"),	NULL,		sub_cb, 6,		"/View/Subtitles/Auto"},
   { N_("/View/Subtitles/Channel _7"),	NULL,		sub_cb, 7,		"/View/Subtitles/Auto"},
   { N_("/_Media"),			NULL,		NULL, 0,		"<Branch>" },
-  { N_("/Media/_Add media mark..."),	"<control>D",	mediamarks_add_cb, 0,	STOCK(NEW) },
-  { N_("/Media/_Manage media marks..."), "<control>B",	mediamarks_manage_cb, 0, NULL },
-  { N_("/Media/_Import media marks..."), NULL,		mediamarks_import_cb, 0, STOCK(OPEN) },
+  { N_("/Media/_Add media mark..."),	"<control>D",	JS(MEDIAMARKS_ADD),	STOCK(NEW) },
+  { N_("/Media/_Manage media marks..."), "<control>B",	JS(MEDIAMARKS_MANAGE),	NULL },
+  { N_("/Media/_Import media marks..."), NULL,		JS(MEDIAMARKS_IMPORT),	STOCK(OPEN) },
   { "/Media/sep8",			NULL,		NULL, 0,		"<Separator>" },
   { N_("/_Help"),			NULL,		NULL, 0,		"<LastBranch>" },
-  { N_("/Help/_About..."),		NULL,		about_cb, 0,		NULL },
-  { N_("/Help/Engine _Log..."),		"<control>L",	log_cb, 0,		NULL },
-  { N_("/Help/Stream _Info..."),	NULL,		stream_info_cb, 0,	NULL },
+  { N_("/Help/_About..."),		NULL,		JS(ABOUT),		STOCKQ(about) },
+  { N_("/Help/Engine _Log..."),		"<control>L",	JS(LOG),		NULL },
+  { N_("/Help/Stream _Info..."),	NULL,		JS(STREAM_INFO),	NULL },
   { "/Help/sep6",			NULL,		NULL, 0,		"<Separator>" },
-  { N_("/Help/_Re-run setup wizards..."), NULL,		wizards_cb, 0,		NULL },
+  { N_("/Help/_Re-run setup wizards..."), NULL,		JS(WIZARDS),		NULL },
 };
 
 GtkWidget *create_menu_tree (GtkItemFactory **factory, GtkWidget *window,
@@ -481,12 +449,27 @@ void create_menus (GtkWidget *window)
     fullscreen_menu_item[i]  = GTK_CHECK_MENU_ITEM (gtk_item_factory_get_item (item_factory[i], "/View/Fullscreen mode"));
 
     deinterlace_menu_item[i] = GTK_CHECK_MENU_ITEM (gtk_item_factory_get_item (item_factory[i], "/View/Deinterlace"));
-    if (xine_config_lookup_entry (xine, "gui.deinterlace_by_default", &entry))
+    if (xine_config_lookup_entry (xine, "gui.post_plugins.deinterlace_enable",
+				  &entry))
       deinterlace_menu_item[i]->active = entry.num_value;
 
-    auto_resize_menu_item[i] = GTK_CHECK_MENU_ITEM (gtk_item_factory_get_item (item_factory[i], "/View/Auto resize"));
+    pp_video_menu_item[i] = GTK_CHECK_MENU_ITEM (gtk_item_factory_get_item (item_factory[i], "/View/Post-processing/Enable video"));
+    if (xine_config_lookup_entry (xine, "gui.post_plugins.video_enable",
+				  &entry))
+      pp_video_menu_item[i]->active = entry.num_value;
+
+    pp_audio_menu_item[i] = GTK_CHECK_MENU_ITEM (gtk_item_factory_get_item (item_factory[i], "/View/Post-processing/Enable audio"));
+    if (xine_config_lookup_entry (xine, "gui.post_plugins.audio_enable",
+				  &entry))
+      pp_audio_menu_item[i]->active = entry.num_value;
+
+    auto_resize_menu_item[i] = GTK_CHECK_MENU_ITEM (gtk_item_factory_get_item (item_factory[i], "/View/Window/Auto resize"));
     auto_resize_menu_item[i]->active = TRUE;
 
+    auto_rescale_menu_item[i] = GTK_CHECK_MENU_ITEM (gtk_item_factory_get_item (item_factory[i], "/View/Window/Magnify low-res video"));
+    if (xine_config_lookup_entry (xine, "gui.magnify_lowres_video", &entry))
+      auto_rescale_menu_item[i]->active = entry.num_value;
+
     toolbar_item[0][i] = GTK_CHECK_MENU_ITEM (gtk_item_factory_get_item (item_factory[i], "/View/Fullscreen toolbar/Visible"));
     toolbar_item[1][i] = GTK_CHECK_MENU_ITEM (gtk_item_factory_get_item (item_factory[i], "/View/Fullscreen toolbar/At bottom"));
     toolbar_item[2][i] = GTK_CHECK_MENU_ITEM (gtk_item_factory_get_item (item_factory[i], "/View/Fullscreen toolbar/At top"));
@@ -507,8 +490,8 @@ void create_menus (GtkWidget *window)
 							       "/File/sep1")));
 
     none_item = GTK_RADIO_MENU_ITEM (gtk_item_factory_get_item (item_factory[i], "/View/Visualisations/None"));
-    add_vis_entries (GTK_MENU_SHELL (gtk_item_factory_get_widget (item_factory[i], "/View/Visualisations")), none_item);
-
+    vis_menu[i] = GTK_MENU_SHELL (gtk_item_factory_get_widget (item_factory[i], "/View/Visualisations"));
+    add_vis_entries (vis_menu[i], none_item);
   }
 
   g_signal_connect (GTK_OBJECT(popup_menu), "key-press-event",
