@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: noskin_window.c,v 1.35 2004/12/15 23:59:59 dsalt Exp $
+ * $Id: noskin_window.c,v 1.42.2.3 2005/05/31 22:13:29 dsalt Exp $
  *
  * standard, non-skinned main window
  */
@@ -45,14 +45,14 @@
 #include "time_widget.h"
 #include "vis.h"
 
-#include "playlist.xpm"
-#include "play.xpm"
-#include "pause.xpm"
-#include "stop.xpm"
-#include "speaker.xpm"
-#include "nospeaker.xpm"
-#include "sliders.xpm"
-#include "fast_forward.xpm"
+#include "play.icn"
+#include "pause.icn"
+#include "fastforward.icn"
+#include "stop.icn"
+#include "playlist.icn"
+#include "settings.icn"
+#include "speaker.icn"
+#include "nospeaker.icn"
 
 /*
 #define VIS_WIDGET
@@ -93,10 +93,23 @@ static void toggle_cb (GtkToggleButton *button, gpointer data)
     ui_set_status (UI_CURRENT_STATE);
 }
 
+static GtkWidget *create_empty_button (gboolean toggle)
+{
+  if (toggle)
+  {
+    GtkWidget *button = gtk_toggle_button_new ();
+    g_signal_connect (GTK_OBJECT(button), "toggled",
+		      G_CALLBACK(toggle_cb), NULL);
+    return button;
+  }
+  return gtk_button_new ();
+}
+
 static GtkWidget *add_pix_button_common (GtkWidget *box, GtkWidget *button,
 					 char *cmd, char *tip, gboolean start)
 {
   gtk_button_set_relief (GTK_BUTTON(button), GTK_RELIEF_NONE);
+
   if (start)
     gtk_box_pack_start (GTK_BOX(box), button, FALSE, FALSE, 2);
   else
@@ -109,45 +122,85 @@ static GtkWidget *add_pix_button_common (GtkWidget *box, GtkWidget *button,
   return button;
 }
 
-static GtkWidget *new_pixmap (gchar **xpm)
+static GtkWidget *new_image (const guint8 image[])
 {
-  GdkPixmap *image;
-  GdkBitmap *transparent;
-
-  image = gdk_pixmap_colormap_create_from_xpm_d
-	    (NULL, gdk_colormap_get_system(), &transparent, NULL, xpm);
-  return gtk_pixmap_new (image, transparent);
+  return gtk_image_new_from_pixbuf
+	   (gdk_pixbuf_new_from_inline (-1, image, FALSE, NULL));
 }
 
-static GtkWidget *add_pix_button (GtkWidget *box, gchar **pixmap_array,
+static GtkWidget *add_pix_button (GtkWidget *box, const guint8 image[],
 				  char *cmd, char *tip, gboolean toggle,
 				  gboolean start)
 {
-  GtkWidget *button;
-
-  if (toggle)
-  {
-    button = gtk_toggle_button_new ();
-    g_signal_connect (GTK_OBJECT(button), "toggled",
-		      G_CALLBACK(toggle_cb), NULL);
-  }
-  else
-    button = gtk_button_new ();
-  gtk_container_add (GTK_CONTAINER(button), new_pixmap (pixmap_array));
+  GtkWidget *button = create_empty_button (toggle);
+  gtk_container_add (GTK_CONTAINER(button), new_image (image));
   return add_pix_button_common (box, button, cmd, tip, start);
 }
 
-static GtkWidget *add_pix_flip_button (GtkWidget *box, gchar **inactive,
-				       gchar **active, char * cmd, char *tip,
-				       gboolean start)
+#ifdef USE_GTK_STOCK_MEDIA
+static GtkWidget *add_pix_button_from_stock (GtkWidget *box, const char *stock,
+					     char *cmd, char *tip,
+					     gboolean toggle, gboolean start)
 {
-  GtkWidget *button = gtk_flip_button_new (new_pixmap (inactive),
-					   new_pixmap (active));
+  GtkWidget *button = create_empty_button (toggle);
+  gtk_container_add (GTK_CONTAINER(button),
+		     gtk_image_new_from_stock (stock,
+					       GTK_ICON_SIZE_SMALL_TOOLBAR));
+  return add_pix_button_common (box, button, cmd, tip, start);
+}
+#endif
+
+static GtkWidget *add_pix_flip_button (GtkWidget *box, const guint8 inactive[],
+				       const guint8 active[], char * cmd,
+				       char *tip, gboolean start)
+{
+  GtkWidget *button = gtk_flip_button_new (new_image (inactive),
+					   new_image (active));
   g_signal_connect (GTK_OBJECT(button), "toggled",
 		    G_CALLBACK(toggle_cb), NULL);
   return add_pix_button_common (box, button, cmd, tip, start);
 }
 
+/*
+ * audio only / vis check
+ */
+
+void window_check_vis (gboolean force)
+{
+  int hv_new = xine_get_stream_info (stream, XINE_STREAM_INFO_HAS_VIDEO);
+  int ha_new = xine_get_stream_info (stream, XINE_STREAM_INFO_HAS_AUDIO);
+
+  if (force || have_video != hv_new || have_audio != ha_new)
+  {
+    have_video = hv_new;
+    have_audio = ha_new;
+    if (have_video) {
+      logprintf ("gxine: stream has video\n");
+      vis_hide (GTK_VIDEO(gtv), &audio_port);
+      gtk_widget_show (gtv);
+    }
+    else if (!have_audio)
+    {
+      logprintf ("gxine: stream has no audio\n");
+      gtk_widget_hide (gtv);
+    }
+    else if (vis_show (GTK_VIDEO(gtv), &audio_port))
+    {
+      logprintf ("gxine: stream has audio (visualisation active)\n");
+      gtk_widget_show (gtv);
+    }
+    else
+    {
+      logprintf ("gxine: stream has audio\n");
+      gtk_widget_hide (gtv);
+    }
+    /* Kludge: resize if the video widget visibility has been changed.
+     * If you have a better way of doing this, submit a patch...
+     */
+    gtk_window_resize (GTK_WINDOW(app), 10, 10);
+  }
+}
+
 /*
  * slider
  */
@@ -155,38 +208,16 @@ static GtkWidget *add_pix_flip_button (GtkWidget *box, gchar **inactive,
 static gint update_slider_cb (gpointer data) {
 
   gint pos_stream, pos_time, length_time;
-  int hv_new, ha_new;
 
-  hv_new = xine_get_stream_info (stream, XINE_STREAM_INFO_HAS_VIDEO);
-  ha_new = xine_get_stream_info (stream, XINE_STREAM_INFO_HAS_AUDIO);
-
-  if (have_video != hv_new || have_audio != ha_new)
+  window_check_vis (FALSE);
+  if (xine_get_status (stream) == XINE_STATUS_PLAY
+      && xine_get_pos_length (stream, &pos_stream, &pos_time, &length_time))
   {
-    have_video = hv_new;
-    have_audio = ha_new;
-    if (have_video) {
-      vis_hide (GTK_VIDEO(gtv), audio_port);
-      gtk_widget_show (gtv);
-      /*gtk_widget_set_size_request (gtv, 480, 205);*/
-    }
-    else if (!have_audio || !vis_show (GTK_VIDEO(gtv), &audio_port))
-      gtk_widget_hide (gtv);
-    else
-      gtk_widget_show (gtv);
-    gtk_window_resize (GTK_WINDOW(app), 10, 10);
+    ui_set_control_adjustment (Control_SEEKER, pos_stream);
+    /* update the control buttons while we're here */
+    ui_set_status (UI_CURRENT_STATE);
   }
 
-  if (xine_get_status (stream) != XINE_STATUS_PLAY)
-    return TRUE;
-
-  if (!xine_get_pos_length (stream, &pos_stream, &pos_time, &length_time))
-    return TRUE;
-
-  ui_set_control_adjustment (Control_SEEKER, pos_stream);
-
-  /* update the control buttons while we're here */
-  ui_set_status (UI_CURRENT_STATE);
-
   return TRUE;
 }
 
@@ -251,16 +282,27 @@ void window_toolbar_restore (void)
     window_toolbar_show_int ();
 }
 
+static gboolean window_toolbar_move (void)
+{
+  gtk_window_move (GTK_WINDOW(controlwindow), 0,
+		   toolbar_at_top ? 1 : gdk_screen_height () - cw_height - 1);
+  return FALSE;
+}
+
 void window_toolbar_position (gboolean top)
 {
   toolbar_at_top = top;
-  gtk_window_move (GTK_WINDOW(controlwindow),
-		   0, top ? 1 : gdk_screen_height () - cw_height - 1);
+  window_toolbar_move ();
   gtk_widget_hide (top ? cw_top : cw_bottom);
   gtk_widget_show (top ? cw_bottom : cw_top);
   ui_set_status (UI_TOOLBAR);
 }
 
+static void cw_hide_cb (GtkWidget *widget)
+{
+  window_toolbar_show (FALSE);
+}
+
 static void cw_top_bottom_cb (GtkWidget *widget, GtkWidget *other)
 {
   window_toolbar_position (widget == cw_top);
@@ -270,10 +312,21 @@ static gboolean window_toolbar_configure_cb (GtkWidget *widget,
 					     GdkEventConfigure *event,
 					     gpointer data)
 {
+  if (cw_height == event->height)
+    return FALSE;
   cw_height = event->height;
+  if (toolbar_visible)
+    g_idle_add ((GSourceFunc) window_toolbar_move, NULL);
   return FALSE;
 }
 
+static gboolean window_toolbar_focus_cb (GtkWidget *widget,
+					 GdkEventFocus *event, gpointer data)
+{
+  gtk_video_grab_focus (GTK_VIDEO (gtv));
+  return TRUE;
+}
+
 static gboolean noskin_main_init_cb (void)
 {
   ui_set_status (xine_get_param (stream, XINE_PARAM_AUDIO_MUTE)
@@ -306,17 +359,26 @@ void noskin_main_init (void) {
   GtkWidget     *audio_spinner;
   time_widget_t *time_widget;
   GdkGeometry    cw_geom;
+  xine_cfg_entry_t entry;
 
   tips = gtk_tooltips_new ();
 
   /* toolbar window (full-screen mode) */
 
   controlwindow = gtk_window_new(GTK_WINDOW_POPUP);
-  g_signal_connect (GTK_OBJECT (controlwindow), "delete_event",
+  g_signal_connect (GTK_OBJECT (controlwindow), "delete-event",
 		    G_CALLBACK (gtk_widget_hide), NULL);
   g_signal_connect (GTK_OBJECT (controlwindow), "configure-event",
 		    G_CALLBACK (window_toolbar_configure_cb), NULL);
-  gtk_window_set_title (GTK_WINDOW(controlwindow), "");
+
+  /* cope with window managers which allow other means of closing windows
+   * and/or which ignore the accept_focus hint
+   */
+  g_signal_connect (G_OBJECT (controlwindow), "unmap-event",
+		    G_CALLBACK (cw_hide_cb), NULL);
+  g_signal_connect (G_OBJECT (controlwindow), "focus-in-event",
+		    G_CALLBACK (window_toolbar_focus_cb), NULL);
+
   gtk_window_set_resizable (GTK_WINDOW(controlwindow), FALSE);
   gtk_window_set_type_hint (GTK_WINDOW(controlwindow),
 			    GDK_WINDOW_TYPE_HINT_TOOLBAR);
@@ -345,7 +407,7 @@ void noskin_main_init (void) {
     w = button_icon_new (GTK_STOCK_CLOSE, _("Hide toolbar"));
     gtk_box_pack_start_defaults (GTK_BOX(vbox), w);
     g_signal_connect_swapped (GTK_OBJECT(w), "clicked",
-		      	      G_CALLBACK(ui_toolbar_toggle), NULL);
+		      	      G_CALLBACK(cw_hide_cb), NULL);
 
     cw_top = button_icon_new (GTK_STOCK_GO_UP, _("Move toolbar to top"));
     gtk_box_pack_start_defaults (GTK_BOX(vbox), cw_top);
@@ -362,11 +424,11 @@ void noskin_main_init (void) {
 
   app = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 
-  g_signal_connect( GTK_OBJECT (app), "delete_event",
+  g_signal_connect( GTK_OBJECT (app), "delete-event",
 		    G_CALLBACK (close_application), NULL );
 
   gtk_widget_add_events (app, GDK_KEY_RELEASE_MASK);
-  g_signal_connect (GTK_OBJECT(app), "key_press_event",
+  g_signal_connect (GTK_OBJECT(app), "key-press-event",
 		    G_CALLBACK(keypress_cb), NULL);
 
   drag_drop_setup (app);
@@ -392,6 +454,14 @@ void noskin_main_init (void) {
   g_signal_connect (GTK_OBJECT (gtv), "keyrelease",
 		    G_CALLBACK (keyrelease_cb), NULL);
 
+  if (xine_config_lookup_entry (xine, "gui.window_size", &entry))
+  {
+    const float scale[] = { 0.5, 1, 2 };
+    gtk_video_rescale ((GtkVideo *)gtv, scale[entry.num_value]);
+  }
+  if (xine_config_lookup_entry (xine, "gui.magnify_lowres_video", &entry))
+    gtk_video_set_auto_rescale ((GtkVideo *)gtv, entry.num_value);
+
   create_menus (app);
   gtk_box_pack_start (GTK_BOX (vbox), menubar, FALSE, FALSE, 0);
 
@@ -424,7 +494,7 @@ void noskin_main_init (void) {
 #endif
 
   time_widget = create_time_widget (&tw);
-  gtk_box_pack_start (GTK_BOX(infobox), tw, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX(infobox), tw, FALSE, TRUE, 0);
   drag_drop_setup (tw);
 
   bar = create_infobar (&infobar);
@@ -456,26 +526,53 @@ void noskin_main_init (void) {
 
   /* buttons */
 
-  ui_register_control_button
-    (Control_PLAY,
-     add_pix_button (ctrl, play_xpm, "play ();", _("Play"), TRUE, TRUE));
-  ui_register_control_button
-    (Control_FASTFWD,
-     add_pix_button (ctrl, fast_forward_xpm,
-		     "if (!is_live_stream ()) set_speed (16);",
-		     _("Fast forward"), TRUE, TRUE));
-  ui_register_control_button
-    (Control_PAUSE,
-     add_pix_button (ctrl, pause_xpm,
-		     "if (!is_live_stream ()) pause ();",
-		     _("Pause"), TRUE, TRUE));
-  ui_register_control_button
-    (Control_STOP,
-     add_pix_button (ctrl, stop_xpm, "stop ();", _("Stop"), TRUE, TRUE));
+#ifdef USE_GTK_STOCK_MEDIA
+  /* We don't want the GTK_STOCK_MEDIA_* just yet. */
+  if (GTK_TEST_COMPAT_2_6)
+  {
+    ui_register_control_button
+      (Control_PLAY,
+       add_pix_button_from_stock (ctrl, "gtk-media-play",
+				  "play ();", _("Play"), TRUE, TRUE));
+    ui_register_control_button
+      (Control_FASTFWD,
+       add_pix_button_from_stock (ctrl, "gtk-media-forward",
+				  "if (!is_live_stream ()) set_speed (16);",
+				  _("Fast forward"), TRUE, TRUE));
+    ui_register_control_button
+      (Control_PAUSE,
+       add_pix_button_from_stock (ctrl, "gtk-media-pause",
+				  "if (!is_live_stream ()) pause ();",
+				  _("Pause"), TRUE, TRUE));
+    ui_register_control_button
+      (Control_STOP,
+       add_pix_button_from_stock (ctrl, "gtk-media-stop",
+				  "stop ();", _("Stop"), TRUE, TRUE));
+  }
+  else
+#endif
+  {
+    ui_register_control_button
+      (Control_PLAY,
+       add_pix_button (ctrl, play_xpm, "play ();", _("Play"), TRUE, TRUE));
+    ui_register_control_button
+      (Control_FASTFWD,
+       add_pix_button (ctrl, fastforward_xpm,
+                       "if (!is_live_stream ()) set_speed (16);",
+                       _("Fast forward"), TRUE, TRUE));
+    ui_register_control_button
+      (Control_PAUSE,
+       add_pix_button (ctrl, pause_xpm,
+                       "if (!is_live_stream ()) pause ();",
+                       _("Pause"), TRUE, TRUE));
+    ui_register_control_button
+      (Control_STOP,
+       add_pix_button (ctrl, stop_xpm, "stop ();", _("Stop"), TRUE, TRUE));
+  }
 
   add_pix_button (ctrl, playlist_xpm,
 		  "playlist_show ()", _("Playlist"), FALSE, TRUE);
-  add_pix_button (ctrl, sliders_xpm,
+  add_pix_button (ctrl, settings_xpm,
 		  "settings_show ()", _("A/V settings"), FALSE, TRUE);
 
   /* audio channel selection */
@@ -513,9 +610,9 @@ void noskin_main_init (void) {
 
   have_video = 1;
 
-  postinit_infobar (app, bar);
-  postinit_time_widget (app, time_widget);
+  postinit_infobar (bar);
+  postinit_time_widget (time_widget);
 
-  gtk_timeout_add (1000, update_slider_cb, NULL);
+  gtk_timeout_add (500, update_slider_cb, NULL);
 }
 #endif /* !EXP_STUFF */
