@@ -16,7 +16,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
  *
- * $Id: player.c,v 1.23 2004/12/11 00:40:18 dsalt Exp $
+ * $Id: player.c,v 1.29.2.2 2005/07/14 23:57:45 dsalt Exp $
  *
  * player object 
  */
@@ -49,7 +49,7 @@ xine_video_port_t *video_port;
 
 static int             goto_pos, goto_time;
 static pthread_t       play_thread;
-static char           *cur_mrl = NULL;
+static char           *cur_mrl = NULL, *cur_title = NULL;
 
 static void *play_exec (void *queue_gen) {
 
@@ -116,6 +116,7 @@ static void *play_exec (void *queue_gen) {
   gdk_threads_enter();
   ui_set_status (playlist_showing_logo () ? UI_STOP : UI_PLAY);
   infobar_show_metadata ();
+  playlist_check_set_title ();
   gdk_threads_leave();
 
 done:
@@ -124,18 +125,18 @@ done:
   return NULL;
 }
 
-void player_launch (const char *mrl, int pos, int pos_time) {
+void player_launch (const char *title, const char *mrl, int pos, int pos_time)
+{
   int err;
 
-  goto_pos  = pos;
-  goto_time = pos_time;
+  goto_pos  = pos < 0 ? 0 : pos;
+  goto_time = pos_time < 0 ? 0 : pos_time;
+  free (cur_title);
   free (cur_mrl);
-  cur_mrl = strdup(mrl);
-
-  /*
-   * start seperate thread, so xine_open will not block ui
-   */
+  cur_title = title && !playlist_showing_logo () ? strdup (title) : NULL;
+  cur_mrl = strdup (mrl);
 
+  /* start separate thread, so xine_open will not block ui */
   if ((err = pthread_create (&play_thread,
 			     NULL, play_exec, NULL)) != 0) {
     printf (_("playlist: can't create new thread (%s)\n"),
@@ -476,10 +477,10 @@ static JSBool js_set_deinterlace (JSContext *cx, JSObject *obj, uintN argc,
     se_arg_is_int (0, "set_deinterlace");
     JS_ValueToInt32 (cx, argv[0], &di);
   } else {
-    di = !xine_get_param (stream, XINE_PARAM_VO_DEINTERLACE);
+    di = !gtk_video_get_deinterlace (GTK_VIDEO(gtv));
   }
 
-  xine_set_param (stream, XINE_PARAM_VO_DEINTERLACE, di);
+  gtk_video_set_deinterlace (GTK_VIDEO(gtv), di);
 
   /* Hmm, should be a don't-emit-signal method for this... */
   deinterlace_menu_item[0]->active = di;
@@ -488,6 +489,83 @@ static JSBool js_set_deinterlace (JSContext *cx, JSObject *obj, uintN argc,
   return JS_TRUE;
 }
 
+static JSBool js_set_postproc_video (JSContext *cx, JSObject *obj,
+				     uintN argc, jsval *argv, jsval *rval)
+{
+  se_t *se = (se_t *) JS_GetContextPrivate(cx);
+  int di;
+
+  se_log_fncall ("set_postproc_video");
+  se_argc_check_max (1, "set_postproc_video");
+
+  if (argc == 1)
+  {
+    if (JSVAL_IS_INT (argv[0]))
+      JS_ValueToInt32 (cx, argv[0], &di);
+    else if (JSVAL_IS_STRING (argv[0]))
+    {
+      gtk_video_set_post_plugins_video
+        (GTK_VIDEO(gtv), JS_GetStringBytes (JS_ValueToString (cx, argv[0])));
+      return JS_TRUE;
+    }
+    else
+    {
+      se->print_cb (se->print_cb_data,
+		   _("error: %s() argument %d is neither int nor string\n"),
+		   "set_postproc_video", 1);
+      return JS_TRUE;
+    }
+  } else
+    di = !gtk_video_get_use_post_plugins_video (GTK_VIDEO(gtv));
+
+  gtk_video_set_use_post_plugins_video (GTK_VIDEO(gtv), di);
+
+  /* Hmm, should be a don't-emit-signal method for this... */
+  pp_video_menu_item[0]->active = di;
+  pp_video_menu_item[1]->active = di;
+
+  return JS_TRUE;
+}
+
+static JSBool js_set_postproc_audio (JSContext *cx, JSObject *obj,
+				     uintN argc, jsval *argv, jsval *rval)
+{
+  se_t *se = (se_t *) JS_GetContextPrivate(cx);
+  int di;
+
+  se_log_fncall ("set_postproc_audio");
+  se_argc_check_max (1, "set_postproc_audio");
+
+  if (argc == 1)
+  {
+    if (JSVAL_IS_INT (argv[0]))
+      JS_ValueToInt32 (cx, argv[0], &di);
+    else if (JSVAL_IS_STRING (argv[0]))
+    {
+      gtk_video_set_post_plugins_audio
+	(GTK_VIDEO(gtv), JS_GetStringBytes (JS_ValueToString (cx, argv[0])),
+	 audio_port);
+      return JS_TRUE;
+    }
+    else
+    {
+      se->print_cb (se->print_cb_data,
+		   _("error: %s() argument %d is neither int nor string\n"),
+		   "set_postproc_audio", 1);
+      return JS_TRUE;
+    }
+  } else
+    di = !gtk_video_get_use_post_plugins_audio (GTK_VIDEO(gtv));
+
+  gtk_video_set_use_post_plugins_audio (GTK_VIDEO(gtv), di, audio_port);
+
+  /* Hmm, should be a don't-emit-signal method for this... */
+  pp_audio_menu_item[0]->active = di;
+  pp_audio_menu_item[1]->active = di;
+
+  return JS_TRUE;
+}
+
 static JSBool js_set_auto_resize (JSContext *cx, JSObject *obj, uintN argc,
 				 jsval *argv, jsval *rval) {
 
@@ -511,6 +589,29 @@ static JSBool js_set_auto_resize (JSContext *cx, JSObject *obj, uintN argc,
   return JS_TRUE;
 }
 
+static JSBool js_set_auto_rescale (JSContext *cx, JSObject *obj, uintN argc,
+				   jsval *argv, jsval *rval)
+{
+  se_t *se = (se_t *) JS_GetContextPrivate(cx);
+  int   ar;
+
+  se_log_fncall ("set_auto_rescale");
+  se_argc_check_max (1, "set_auto_rescale");
+
+  if (argc==1) {
+    se_arg_is_int (0, "set_auto_rescale");
+    JS_ValueToInt32 (cx, argv[0], &ar);
+  } else {
+    ar = !gtk_video_get_auto_rescale (GTK_VIDEO(gtv));
+  }
+
+  gtk_video_set_auto_rescale (GTK_VIDEO(gtv), ar);
+  auto_rescale_menu_item[0]->active = ar;
+  auto_rescale_menu_item[1]->active = ar;
+
+  return JS_TRUE;
+}
+
 static JSBool js_set_aspect (JSContext *cx, JSObject *obj, uintN argc, 
 			     jsval *argv, jsval *rval) {
 
@@ -706,49 +807,30 @@ static JSBool controls_play (JSContext *cx, JSObject *obj, uintN argc,
     break;
   }
 
-  pos *=655 ;
+  if (pos > 0)
+    pos *= 655;
 
   if (engine_mutex_trylock (se))
     return JS_TRUE;
 
-  if (mrl) {
-
-    int item;
-
-    item = playlist_add_mrl (mrl, -1);
-
-    if (pos<0)
-      pos = 0;
-    if (pos_time<0)
-      pos_time = 0;
-
-    playlist_play_from (item, pos, pos_time);
-
-  } else {
-
-    play_item_t *item;
-
-    item = playlist_get_item (playlist_get_list_pos());
+  if (mrl)
+    playlist_play_from (playlist_add_mrl (mrl, -1), pos, pos_time);
+  else
+  {
+    play_item_t *item = playlist_get_item (playlist_get_list_pos());
 
     if (item) {
-      if (xine_get_status (stream) == XINE_STATUS_STOP) {
-
-	if (pos<0)
-	  pos = 0;
-	if (pos_time<0)
-	  pos_time = item->start_time;
-	
-	playlist_play_from (playlist_get_list_pos(), pos, pos_time);
-      } else {
-	
-	if ( (pos>=0) || (pos_time>=0) ) {
-	  
-	  if (pos<0)
-	    pos = 0;
-	  if (pos_time<0)
-	    pos_time = 0;
-	  xine_play (stream, pos, pos_time);
-	}
+      switch (xine_get_status (stream))
+      {
+      case XINE_STATUS_STOP:
+      case XINE_STATUS_IDLE:
+	playlist_play_from (playlist_get_list_pos (), pos,
+			   pos_time < 0 ? item->start_time : pos_time);
+	break;
+      default:
+	if (pos >= 0 || pos_time >= 0)
+	  xine_play (stream, pos < 0 ? 0 : pos, pos_time < 0 ? 0 : pos_time);
+	break;
       }
     } else
       printf (_("script_engine: error, no valid play item available\n"));
@@ -779,10 +861,14 @@ static JSBool controls_play_pause (JSContext *cx, JSObject *obj, uintN argc,
     se_arg_is_int (0, "play_pause");
   }
 
-  if (xine_get_status (stream) == XINE_STATUS_STOP)
+  switch (xine_get_status (stream))
+  {
+  case XINE_STATUS_STOP:
+  case XINE_STATUS_IDLE:
     return controls_play (cx, obj, 0, 0, rval);
-
-  return controls_pause (cx, obj, argc, argv, rval);
+  default:
+    return controls_pause (cx, obj, argc, argv, rval);
+  }
 }
 
 static JSBool js_is_live_stream (JSContext *cx, JSObject *obj, uintN argc,
@@ -807,13 +893,20 @@ int player_live_stream (void)
 
   return (xine_get_status (stream) == XINE_STATUS_PLAY) &&
 	 (xine_get_pos_length (stream, &pos_stream, &pos_time, &length_time)) &&
-	 length_time == 0;
+	 (pos_stream | length_time) == 0;
 }
 
-char *player_get_cur_mrl (void) {
+const char *player_get_cur_title (void)
+{
+  return cur_title;
+}
+
+const char *player_get_cur_mrl (void)
+{
   return cur_mrl;
 }
 
+
 static xine_audio_port_t *load_audio_out_driver (void) {
  
   xine_audio_port_t      *audio_port;
@@ -901,7 +994,7 @@ void player_init (void) {
 	SE_GROUP_ENGINE, N_("[file name]"), NULL },
 
       { "set_speed", js_set_speed, 0, 0,
-	SE_GROUP_PROPERTIES, N_("[bool]"), NULL },
+	SE_GROUP_PROPERTIES, N_("int"), NULL },
       { "get_speed", js_get_speed, 0, 0,
 	SE_GROUP_PROPERTIES, NULL, NULL },
       { "set_volume", js_set_volume, 0, 0,
@@ -926,8 +1019,14 @@ void player_init (void) {
 	SE_GROUP_PROPERTIES, N_("int"), NULL },
       { "set_deinterlace", js_set_deinterlace, 0, 0,
 	SE_GROUP_PROPERTIES, N_("[bool]"), NULL },
+      { "set_postproc_video", js_set_postproc_video, 0, 0,
+	SE_GROUP_PROPERTIES, N_("[bool|string]"), NULL },
+      { "set_postproc_audio", js_set_postproc_audio, 0, 0,
+	SE_GROUP_PROPERTIES, N_("[bool|string]"), NULL },
       { "set_auto_resize", js_set_auto_resize, 0, 0,
 	SE_GROUP_PROPERTIES, N_("[bool]"), NULL },
+      { "set_auto_rescale", js_set_auto_rescale, 0, 0,
+	SE_GROUP_PROPERTIES, N_("[bool]"), N_("magnify low-res video") },
 
       { "is_live_stream", js_is_live_stream, 0, 0,
 	SE_GROUP_PROPERTIES, NULL, NULL },
@@ -935,4 +1034,21 @@ void player_init (void) {
     };
     se_defuns (gse, NULL, defs);
   }
+
+  {
+    static char *window_size_labels[] = {
+      N_("50%"), N_("100%"), N_("200%"),
+      NULL
+    };
+    xine_config_register_enum
+      (xine, "gui.window_size", 0, window_size_labels,
+       _("The main window's default size"),
+       NULL, 0, NULL, NULL);
+    xine_config_register_bool
+      (xine, "gui.magnify_lowres_video", 0,
+       _("Double the size of small video streams"),
+       _("This affects video streams whose dimensions are at most ⅓ "
+	 "of those of the display."),
+       0, NULL, NULL);
+  }
 }
