@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2003 the xine project
+ * Copyright (C) 2001-2005 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: preferences.c,v 1.36 2004/12/14 18:20:32 dsalt Exp $
+ * $Id: preferences.c,v 1.44 2005/01/09 20:56:44 dsalt Exp $
  *
  * Functions to stup and deal with a preferences dialog interfacing with
  * the xine config system.
@@ -30,6 +30,7 @@
 #include <X11/Xlib.h>
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
+#include <gdk/gdkkeysyms.h>
 #include <glib.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -41,9 +42,13 @@
 
 #define XP_UNIX
 #ifdef HAVE_LIBSMJS
-#include <smjs/jsapi.h>
+# ifdef LIBJS_IS_SMJS
+#  include <js/jsapi.h>
+# else
+#  include <smjs/jsapi.h>
+# endif
 #else
-#include "spidermonkey/jsapi.h"
+# include "spidermonkey/jsapi.h"
 #endif
 
 
@@ -130,6 +135,14 @@ static int select_show_prefs_internal (GNode *page, int exp)
     gtk_notebook_append_page (GTK_NOTEBOOK(parent->notebook), item->page.page,
 			      gtk_label_new (item->page.label));
 
+  if (parent)
+  {
+    gboolean multi =
+      gtk_notebook_get_n_pages (GTK_NOTEBOOK(parent->notebook)) > 1;
+    gtk_notebook_set_show_tabs (GTK_NOTEBOOK(parent->notebook), multi);
+    gtk_notebook_set_show_border (GTK_NOTEBOOK(parent->notebook), multi);
+  }
+
   return state;
 }
 
@@ -152,7 +165,7 @@ static gboolean select_show_pref_widgets (void *data)
   return FALSE;
 }
 
-static gboolean entry_cb (GtkEditable *editable, GdkEventFocus *even,
+static gboolean entry_cb (GtkEntry *editable, GdkEventFocus *even,
 			  gpointer user_data)
 {
   xine_cfg_entry_t      entry;
@@ -166,7 +179,7 @@ static gboolean entry_cb (GtkEditable *editable, GdkEventFocus *even,
   if (!xine_config_lookup_entry (xine, key, &entry))
     return FALSE;
 
-  key = gtk_editable_get_chars (editable, 0, -1);
+  key = gtk_editable_get_chars (GTK_EDITABLE(editable), 0, -1);
   if (!strcmp (entry.str_value, key))
     return FALSE;
 
@@ -177,29 +190,28 @@ static gboolean entry_cb (GtkEditable *editable, GdkEventFocus *even,
   return FALSE;
 }
 
-static gboolean num_entry_cb (GtkEditable *editable, GdkEventFocus *even,
-			      gpointer user_data)
+static gboolean entry_keypress_cb (GtkEntry *widget, GdkEventKey *event,
+				   gpointer data)
 {
-  xine_cfg_entry_t      entry;
-  gchar                *key = (gchar *) user_data;
-  gchar                *str;
-  int                   num;
+  switch (event->keyval)
+  {
+  case GDK_Return:
+  case GDK_KP_Enter:
+    entry_cb (widget, NULL, data);
+    return TRUE;
 
-  if (update_lock)
-    return FALSE;
-
-  logprintf ("preferences: num entry cb for key %s\n", key);
-
-  if (!xine_config_lookup_entry (xine, key, &entry))
-    return FALSE;
-
-  str = gtk_editable_get_chars (editable, 0, -1);
-  if ( sscanf (str, "%d", &num) != 1 || num == entry.num_value)
-    return FALSE;
-
-  logprintf ("preferences: updating entry to %d\n", num);
-  entry.num_value = num;
-  xine_config_update_entry (xine, &entry);
+  case GDK_minus:
+    if ((event->state & GXINE_MODIFIER_MASK) != GDK_CONTROL_MASK)
+      break;
+  case GDK_Undo:
+    {
+      const char *key = g_object_get_data (G_OBJECT(widget), "cfg");
+      xine_cfg_entry_t entry;
+      if (key && xine_config_lookup_entry (xine, key, &entry))
+	gtk_entry_set_text (widget, entry.str_value);
+    }
+    return TRUE;
+  }
 
   return FALSE;
 }
@@ -227,10 +239,9 @@ static void check_box_cb (GtkToggleButton *togglebutton, gpointer user_data)
   xine_config_update_entry (xine, &entry);
 }
 
-static void range_cb (GtkWidget* widget, gpointer data) {
-
+static void range_cb (GtkAdjustment *adj, const gchar *key)
+{
   xine_cfg_entry_t      entry;
-  gchar                *key = (gchar *) data;
 
   if (update_lock)
     return;
@@ -238,11 +249,31 @@ static void range_cb (GtkWidget* widget, gpointer data) {
   logprintf ("preferences: range cb for key %s\n", key);
 
   if (!xine_config_lookup_entry (xine, key, &entry) ||
-      entry.num_value == GTK_ADJUSTMENT(widget)->value)
+      entry.num_value == adj->value)
     return;
 
-  logprintf ("preferences: updating entry to %d\n", entry.num_value);
-  entry.num_value = GTK_ADJUSTMENT(widget)->value;
+  logprintf ("preferences: updating entry to %lf\n", adj->value);
+  entry.num_value = adj->value;
+  xine_config_update_entry (xine, &entry);
+}
+
+static void spin_cb (GtkSpinButton *widget, const gchar *key)
+{
+  xine_cfg_entry_t entry;
+  int value;
+
+  if (update_lock)
+    return;
+
+  logprintf ("preferences: spin cb for key %s\n", key);
+
+  value = gtk_spin_button_get_value_as_int (widget);
+  if (!xine_config_lookup_entry (xine, key, &entry) ||
+      entry.num_value == value)
+    return;
+
+  logprintf ("preferences: updating entry to %d\n", value);
+  entry.num_value = value;
   xine_config_update_entry (xine, &entry);
 }
 
@@ -296,7 +327,10 @@ static GtkWidget *create_item_editable (const xine_cfg_entry_t *entry)
 
   case XINE_CONFIG_TYPE_STRING:
     widget = gtk_entry_new();
-    g_signal_connect (GTK_OBJECT(widget), "focus-out-event", 
+    g_object_set_data (G_OBJECT(widget), "cfg", (gpointer)entry->key);
+    g_signal_connect (GTK_OBJECT(widget), "key-press-event",
+		      G_CALLBACK(entry_keypress_cb), (gchar *)entry->key);
+    g_signal_connect (GTK_OBJECT(widget), "focus-out-event",
 		      G_CALLBACK(entry_cb), (gchar *)entry->key);
     gtk_entry_set_text (GTK_ENTRY(widget), entry->str_value);
     break;
@@ -309,19 +343,21 @@ static GtkWidget *create_item_editable (const xine_cfg_entry_t *entry)
       gtk_scale_set_draw_value (GTK_SCALE(widget), TRUE);
       gtk_scale_set_value_pos (GTK_SCALE(widget), GTK_POS_TOP);
       gtk_scale_set_digits (GTK_SCALE(widget), 0);
-      g_signal_connect (GTK_OBJECT (adj), "value-changed",
+      g_signal_connect (adj, "value-changed",
 			G_CALLBACK (range_cb), (gchar *)entry->key);
     }
     break;
 
   case XINE_CONFIG_TYPE_NUM:
     {
-      char contents[20];
-      widget = gtk_entry_new();
-      snprintf (contents, sizeof (contents), "%i", entry->num_value);
-      g_signal_connect (GTK_OBJECT(widget), "focus-out-event",
-			G_CALLBACK(num_entry_cb), (gchar *)entry->key);
-      gtk_entry_set_text (GTK_ENTRY(widget), contents);
+      GtkObject *adj = gtk_adjustment_new (entry->num_value, INT_MIN, INT_MAX,
+					   1, 10, 0);
+      widget = gtk_spin_button_new (GTK_ADJUSTMENT(adj), 1, 0);
+      gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON(widget),
+					 GTK_UPDATE_ALWAYS);
+      gtk_spin_button_set_numeric (GTK_SPIN_BUTTON(widget), TRUE);
+      g_signal_connect (GTK_OBJECT(widget), "value-changed",
+			G_CALLBACK(spin_cb), (gchar *)entry->key);
     }
     break;
 
@@ -592,6 +628,8 @@ static void response_cb (GtkDialog *dbox, int response, gpointer data)
   gchar *fname;
   switch (response)
   {
+  case GTK_RESPONSE_REJECT:
+    
   case GTK_RESPONSE_ACCEPT:
     fname = g_strconcat(g_get_home_dir(), "/.gxine/config", NULL);
     xine_config_save (xine, fname);
