@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2003 the xine-project
+ * Copyright (C) 2002-2004 the xine-project
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -16,90 +16,37 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
  *
- * $Id: skin_window.c,v 1.5 2003/03/14 20:47:05 guenter Exp $
+ * $Id: skin_window.c,v 1.17.2.1 2005/03/17 01:11:30 dsalt Exp $
  *
  * cool fully-user-skinned windows, generate from xml/js code
  */
 
+#include "globals.h"
+
+#ifdef EXP_STUFF
+
 #include <config.h>
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <inttypes.h>
+#include <errno.h>
 
 #include <gdk/gdk.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 #include <X11/Xlib.h>
+#include "xmlparser.h"
 
 #include "skin_window.h"
 #include "utils.h"
-#include "actions.h"
-#include "xmlparser.h"
+#include "script_engine.h"
+#include "skin.h"
+#include "widget_video.h"
 
-typedef struct view_s view_t;
-
-struct view_s {
-
-  char       *basename;
-
-  char       *id;
-  int         w, h;
-
-  gint        x, y;
-  gboolean    is_moving;
-  gboolean    repaint_needed;
-
-  GList      *widgets;
-
-  GtkWidget  *window;
-  GdkPixmap  *bg;
-  GdkBitmap  *mask;
-};
-
-#define WT_SUBVIEW     0
-#define WT_BUTTON      1
-#define WT_BUTTONGROUP 2
-#define WT_SLIDER      3
-
-typedef struct skin_widget_s skin_widget_t;
-
-struct skin_widget_s {
-
-  view_t    *parent;
-
-  char      *id;
-
-  char      *xscript, *yscript;
-  int        x, y, w, h;
-  int        zIndex;
-  int        type;
-
-  void (*layout)             (skin_widget_t *w, int xo, int yo);
-  void (*paint)              (skin_widget_t *w);
-  void (*handle_motion)      (skin_widget_t *w, int px, int py);
-  gboolean (*handle_button_press) (skin_widget_t *w, int px, int py);
-  void (*handle_button_release)   (skin_widget_t *w, int px, int py);
-};
-
-typedef struct subview_s subview_t;
-
-struct subview_s {
-
-  skin_widget_t widget;
-
-  gboolean   tiled;
-  char      *halign, *valign;
-  /* tiled borders limits: */
-  char      *h_max_script, *v_max_script;
-  int        h_max, v_max;
-
-  GdkPixbuf *pix;
-
-  GList     *widgets;
-};
+GtkWidget *gtv = NULL;
 
 typedef struct buttongroup_s buttongroup_t;
 
@@ -113,6 +60,19 @@ struct buttongroup_s {
   GList         *buttons;
 };
 
+typedef struct button_s button_t;
+
+struct button_s {
+
+  skin_widget_t  widget;
+
+  GdkPixbuf     *pix, *pix_hover, *pix_down, *pix_hover_down;
+
+  gboolean       pressed, hovered;
+
+  char          *on_click;
+};
+
 typedef struct {
 
   buttongroup_t *parent;
@@ -127,30 +87,40 @@ typedef struct {
 
 } button_element_t;
 
+
 #define MAXPATH 1024
 
-static int script_view_cb (void *view_gen, char *prop) {
-
-  view_t *view = (view_t *) view_gen;
-
-  if (!strcasecmp (prop, "width")) {
-    return view->w;
-  } else if (!strcasecmp (prop, "height")) {
-    return view->h;
-  }
-
-  printf ("skin_window: error, unknown prop %s\n", prop);
-  return -1;
+static void print_cb (void *user_data, char *str, ...)
+{
+  va_list ap;
+  printf ("skin_window: < ");
+  va_start (ap, str);
+  vprintf (str, ap);
+  puts ("*********************************** ERROR\n");
+  abort();
 }
 
-static int se_eval_int (char *script, view_t *view) {
+static int se_eval_int (char *script, se_o_t *o) {
 
   if (!script)
     return 0;
 
-  if (!strncasecmp (script, "jscript:", 8)) 
-    return action_eval_view (&script[8], script_view_cb, view);
-  else 
+  logprintf ("skin_window: evaluating '%s'\n", script);
+
+  if (!strncasecmp (script, "jscript:", 8)) {
+
+    int num;
+
+    se_eval (gse, script, o, print_cb, NULL);
+
+    if ( se_result_num (gse, &num) ) {
+      logprintf ("skin_window: result: %d\n", num);
+      return num;
+    } else
+      printf ("skin_window: error, non-numeric result\n");
+
+    return 0;
+  } else 
     return atoi (script);
 }
 
@@ -410,9 +380,9 @@ static button_element_t *create_button_element (xml_node_t *xbutton_element,
  *
  */
 
-static void buttongroup_paint (skin_widget_t *w) {
-  buttongroup_t *bg = (buttongroup_t *) w;
-  view_t    *view = bg->widget.parent;
+static void buttongroup_paint (skin_widget_t *w, int xo, int yo) {
+  buttongroup_t  *bg = (buttongroup_t *) w;
+  view_t         *view = bg->widget.view;
 
   if (bg->pix) {
 
@@ -431,32 +401,32 @@ static void buttongroup_paint (skin_widget_t *w) {
 	  if (b->pix_hover_down) {
 	    printf ("using hover down image\n");
 	    gdk_pixbuf_render_to_drawable (b->pix_hover_down, view->bg, NULL, 0, 0,
-					   bg->widget.x, 
-					   bg->widget.y,
+					   bg->widget.x+xo, 
+					   bg->widget.y+yo,
 					   -1, -1, GDK_RGB_DITHER_MAX, 0, 0);
 	  } else 
 #endif
 	    if (b->pix_down)
 	    gdk_pixbuf_render_to_drawable (b->pix_down, view->bg, NULL, 0, 0,
-					   bg->widget.x, 
-					   bg->widget.y,
+					   bg->widget.x+xo, 
+					   bg->widget.y+yo,
 					   -1, -1, GDK_RGB_DITHER_MAX, 0, 0);
 	  else if (b->pix)
 	    gdk_pixbuf_render_to_drawable (b->pix, view->bg, NULL, 0, 0,
-					   bg->widget.x, 
-					   bg->widget.y,
+					   bg->widget.x+xo, 
+					   bg->widget.y+yo,
 					   -1, -1, GDK_RGB_DITHER_MAX, 0, 0);
 	} else { /* !pressed */
 
 	  if (b->pix_hover)
 	    gdk_pixbuf_render_to_drawable (b->pix_hover, view->bg, NULL, 0, 0,
-					   bg->widget.x, 
-					   bg->widget.y,
+					   bg->widget.x+xo, 
+					   bg->widget.y+yo,
 					   -1, -1, GDK_RGB_DITHER_MAX, 0, 0);
 	  else if (b->pix)
 	    gdk_pixbuf_render_to_drawable (b->pix, view->bg, NULL, 0, 0,
-					   bg->widget.x, 
-					   bg->widget.y,
+					   bg->widget.x+xo, 
+					   bg->widget.y+yo,
 					   -1, -1, GDK_RGB_DITHER_MAX, 0, 0);
 	}
 
@@ -465,21 +435,21 @@ static void buttongroup_paint (skin_widget_t *w) {
 	if (b->pressed) {
 	  if (b->pix_down)
 	    gdk_pixbuf_render_to_drawable (b->pix_down, view->bg, NULL, 0, 0,
-					   bg->widget.x, 
-					   bg->widget.y,
+					   bg->widget.x+xo, 
+					   bg->widget.y+yo,
 					   -1, -1, GDK_RGB_DITHER_MAX, 0, 0);
 	  else if (b->pix)
 	    gdk_pixbuf_render_to_drawable (b->pix, view->bg, NULL, 0, 0,
-					   bg->widget.x, 
-					   bg->widget.y,
+					   bg->widget.x+xo, 
+					   bg->widget.y+yo,
 					   -1, -1, GDK_RGB_DITHER_MAX, 0, 0);
 	  
 	} else { /* !pressed */
 
 	  if (b->pix)
 	    gdk_pixbuf_render_to_drawable (b->pix, view->bg, NULL, 0, 0,
-					   bg->widget.x, 
-					   bg->widget.y,
+					   bg->widget.x+xo, 
+					   bg->widget.y+yo,
 					   -1, -1, GDK_RGB_DITHER_MAX, 0, 0);
 	}
 
@@ -490,12 +460,12 @@ static void buttongroup_paint (skin_widget_t *w) {
   }
 }
 
-static void buttongroup_layout (skin_widget_t *w, int xo, int yo) {
+static void buttongroup_layout (skin_widget_t *w) {
   buttongroup_t *bg = (buttongroup_t *) w;
 
   if (bg->pix) {
-    bg->widget.x  = se_eval_int (bg->widget.xscript, bg->widget.parent) + xo;
-    bg->widget.y  = se_eval_int (bg->widget.yscript, bg->widget.parent) + yo;
+    bg->widget.x  = se_eval_int (bg->widget.xscript, w->se_obj);
+    bg->widget.y  = se_eval_int (bg->widget.yscript, w->se_obj);
   } 
 }
 
@@ -517,14 +487,14 @@ static gboolean buttongroup_handle_button_press (skin_widget_t *w, int px, int p
 
       if (!b->pressed) {
 	b->pressed = TRUE;
-	bg->widget.parent->repaint_needed = TRUE;
+	bg->widget.view->repaint_needed = TRUE;
       }
 
     } else {
       
       if (b->pressed) {
 	b->pressed = FALSE;
-	bg->widget.parent->repaint_needed = TRUE;
+	bg->widget.view->repaint_needed = TRUE;
       }
     }
     
@@ -550,15 +520,17 @@ static void buttongroup_handle_button_release (skin_widget_t *w, int px, int py)
     
     if (is_inside_group && is_inside_map (bg->pix_map, b->color, px-w->x, py-w->y)) {
 
-      printf ("skin_window: action: %s\n", b->script);
+      printf ("button from buttongroup released\n");
 
-      action_exec (b->script, NULL, NULL);
+      printf ("executing '%s' ...\n", b->script);
+
+      se_eval (gse, b->script, w->se_obj, print_cb, NULL);
 
     }
 
     if (b->pressed) {
       b->pressed = FALSE;
-      bg->widget.parent->repaint_needed = TRUE;
+      bg->widget.view->repaint_needed = TRUE;
     }
     
     n = g_list_next (n);
@@ -583,14 +555,14 @@ static void buttongroup_handle_motion (skin_widget_t *w, int px, int py) {
 
       if (!b->hovered) {
 	b->hovered = TRUE;
-	bg->widget.parent->repaint_needed = TRUE;
+	bg->widget.view->repaint_needed = TRUE;
       }
       
     } else {
       
       if (b->hovered) {
 	b->hovered = FALSE;
-	bg->widget.parent->repaint_needed = TRUE;
+	bg->widget.view->repaint_needed = TRUE;
       }
       
     }
@@ -599,14 +571,15 @@ static void buttongroup_handle_motion (skin_widget_t *w, int px, int py) {
   }
 }
 
-static buttongroup_t *create_buttongroup (xml_node_t *xbuttongroup, view_t *view) {
+static buttongroup_t *create_buttongroup (xml_node_t *xbuttongroup, skin_widget_t *parent) {
 
   buttongroup_t *buttongroup;
   xml_node_t    *node;
 
   buttongroup = (buttongroup_t *) malloc (sizeof (buttongroup_t));
 
-  buttongroup->widget.parent  = view;
+  buttongroup->widget.parent  = parent;
+  buttongroup->widget.view    = parent->view;
   buttongroup->widget.id      = xml_parser_get_property     (xbuttongroup, "id");
   buttongroup->widget.zIndex  = xml_parser_get_property_int (xbuttongroup, "zIndex", 0);
 
@@ -619,12 +592,12 @@ static buttongroup_t *create_buttongroup (xml_node_t *xbuttongroup, view_t *view
   buttongroup->widget.handle_button_press   = buttongroup_handle_button_press;
   buttongroup->widget.handle_button_release = buttongroup_handle_button_release;
 
-  buttongroup->pix       = load_pix_trans (xbuttongroup, "image", view->basename);
-  buttongroup->pix_hover = load_pix_trans (xbuttongroup, "hoverImage", view->basename);
-  buttongroup->pix_down  = load_pix_trans (xbuttongroup, "downImage", view->basename);
+  buttongroup->pix       = load_pix_trans (xbuttongroup, "image", parent->view->basename);
+  buttongroup->pix_hover = load_pix_trans (xbuttongroup, "hoverImage", parent->view->basename);
+  buttongroup->pix_down  = load_pix_trans (xbuttongroup, "downImage", parent->view->basename);
   buttongroup->pix_hover_down = load_pix_trans (xbuttongroup, "hoverDownImage", 
-						view->basename);
-  buttongroup->pix_map   = load_pix (xbuttongroup, "mappingImage", view->basename);
+						parent->view->basename);
+  buttongroup->pix_map   = load_pix (xbuttongroup, "mappingImage", parent->view->basename);
 
   if (buttongroup->pix) {
     buttongroup->widget.w = gdk_pixbuf_get_width  (buttongroup->pix);
@@ -636,10 +609,21 @@ static buttongroup_t *create_buttongroup (xml_node_t *xbuttongroup, view_t *view
 
   buttongroup->buttons = NULL;
 
+  if (!buttongroup->widget.id)
+    buttongroup->widget.id = unique_name ("buttongroup");
+
+  buttongroup->widget.se_obj = se_create_object (gse, parent->se_obj,
+						 buttongroup->widget.id,
+						 buttongroup);
+
   node = xbuttongroup->child;
   while (node) {
 
-    if (!strcasecmp (node->name, "buttonelement")) {
+    if (!strcasecmp (node->name, "buttonelement")
+	|| !strcasecmp (node->name, "playelement")
+	|| !strcasecmp (node->name, "nextelement")
+	|| !strcasecmp (node->name, "stopelement")
+	|| !strcasecmp (node->name, "prevelement")) {
 
       button_element_t *button_element;
 
@@ -655,23 +639,191 @@ static buttongroup_t *create_buttongroup (xml_node_t *xbuttongroup, view_t *view
   return buttongroup;
 }
 
+/*
+ *
+ * button stuff
+ *
+ */
+
+static void button_paint (skin_widget_t *w, int xo, int yo) {
+  button_t  *b = (button_t *) w;
+  view_t    *view = b->widget.view;
+
+  if (b->hovered) {
+	
+    if (b->pressed) {
+      if (b->pix_hover_down) {
+	gdk_pixbuf_render_to_drawable (b->pix_hover_down, view->bg, NULL, 0, 0,
+				       b->widget.x+xo, 
+				       b->widget.y+yo,
+				       -1, -1, GDK_RGB_DITHER_MAX, 0, 0);
+      } else 
+	if (b->pix_down)
+	  gdk_pixbuf_render_to_drawable (b->pix_down, view->bg, NULL, 0, 0,
+					 b->widget.x+xo, 
+					 b->widget.y+yo,
+					 -1, -1, GDK_RGB_DITHER_MAX, 0, 0);
+	else if (b->pix)
+	  gdk_pixbuf_render_to_drawable (b->pix, view->bg, NULL, 0, 0,
+					 b->widget.x+xo, 
+					 b->widget.y+yo,
+					 -1, -1, GDK_RGB_DITHER_MAX, 0, 0);
+    } else { /* !pressed */
+      
+      if (b->pix_hover)
+	gdk_pixbuf_render_to_drawable (b->pix_hover, view->bg, NULL, 0, 0,
+				       b->widget.x+xo, 
+				       b->widget.y+yo,
+				       -1, -1, GDK_RGB_DITHER_MAX, 0, 0);
+      else if (b->pix)
+	gdk_pixbuf_render_to_drawable (b->pix, view->bg, NULL, 0, 0,
+				       b->widget.x+xo, 
+				       b->widget.y+yo,
+				       -1, -1, GDK_RGB_DITHER_MAX, 0, 0);
+    }
+    
+  } else { /* !hovered */
+    
+    if (b->pressed) {
+      if (b->pix_down)
+	gdk_pixbuf_render_to_drawable (b->pix_down, view->bg, NULL, 0, 0,
+				       b->widget.x+xo, 
+				       b->widget.y+yo,
+				       -1, -1, GDK_RGB_DITHER_MAX, 0, 0);
+      else if (b->pix)
+	gdk_pixbuf_render_to_drawable (b->pix, view->bg, NULL, 0, 0,
+				       b->widget.x+xo, 
+				       b->widget.y+yo,
+				       -1, -1, GDK_RGB_DITHER_MAX, 0, 0);
+      
+    } else { /* !pressed */
+      
+      if (b->pix)
+	gdk_pixbuf_render_to_drawable (b->pix, view->bg, NULL, 0, 0,
+				       b->widget.x+xo, 
+				       b->widget.y+yo,
+				       -1, -1, GDK_RGB_DITHER_MAX, 0, 0);
+    }
+  }
+}
+
+static void button_layout (skin_widget_t *w) {
+  button_t *button = (button_t *) w;
+
+  button->widget.x  = se_eval_int (w->xscript, w->se_obj);
+  button->widget.y  = se_eval_int (w->yscript, w->se_obj);
+}
+
+static gboolean button_handle_button_press (skin_widget_t *w, int px, int py) {
+
+  button_t *button = (button_t *) w;
+  gboolean       is_inside_group;
+
+  is_inside_group = is_inside (px, py, w->x, w->y, w->w, w->h);
+
+  if (button->pressed != is_inside_group) {
+    w->view->repaint_needed = TRUE;
+    button->pressed = is_inside_group;
+  }
+
+  return is_inside_group;
+}
+
+static void button_handle_button_release (skin_widget_t *w, int px, int py) {
+
+  button_t *button = (button_t *) w;
+  gboolean       is_inside_group;
+
+  is_inside_group = is_inside (px, py, w->x, w->y, w->w, w->h);
+
+  if (is_inside_group) {
+    if (button->pressed) {
+      
+      button->pressed = FALSE;
+      w->view->repaint_needed = TRUE;
+
+      printf ("button '%s' released\n", w->id);
+
+      printf ("executing '%s' ...\n", button->on_click);
+      
+      se_eval (gse, button->on_click, w->se_obj, print_cb, NULL);
+    }
+  }
+}
+
+static void button_handle_motion (skin_widget_t *w, int px, int py) {
+
+  button_t      *button = (button_t *) w;
+  gboolean       is_inside_group;
+
+  is_inside_group= is_inside (px, py, w->x, w->y, w->w, w->h);
+
+  if (button->hovered != is_inside_group) {
+    w->view->repaint_needed = TRUE;
+    button->hovered = is_inside_group;
+  }
+}
+
+static button_t *create_button (xml_node_t *xbutton, skin_widget_t *parent) {
+
+  button_t *button;
+
+  button = (button_t *) malloc (sizeof (button_t));
+
+  button->widget.parent  = parent;
+  button->widget.view    = parent->view;
+  button->widget.id      = xml_parser_get_property     (xbutton, "id");
+  button->on_click       = xml_parser_get_property     (xbutton, "onClick");
+  button->widget.zIndex  = xml_parser_get_property_int (xbutton, "zIndex", 0);
+
+  button->widget.xscript = xml_parser_get_property     (xbutton, "left");
+  button->widget.yscript = xml_parser_get_property     (xbutton, "top");
+
+  button->widget.layout                = button_layout;
+  button->widget.paint                 = button_paint;
+  button->widget.handle_motion         = button_handle_motion;
+  button->widget.handle_button_press   = button_handle_button_press;
+  button->widget.handle_button_release = button_handle_button_release;
+
+  button->pix       = load_pix_trans (xbutton, "image", parent->view->basename);
+  button->pix_hover = load_pix_trans (xbutton, "hoverImage", parent->view->basename);
+  button->pix_down  = load_pix_trans (xbutton, "downImage", parent->view->basename);
+  button->pix_hover_down = load_pix_trans (xbutton, "hoverDownImage", 
+					   parent->view->basename);
+
+  button->hovered = FALSE;
+  button->pressed = FALSE;
+
+  if (button->pix) {
+    button->widget.w = gdk_pixbuf_get_width  (button->pix);
+    button->widget.h = gdk_pixbuf_get_height (button->pix);
+  } else {
+    button->widget.w = 0;
+    button->widget.h = 0;
+  }
+
+  if (!button->widget.id)
+    button->widget.id = unique_name ("button");
+
+  button->widget.se_obj = se_create_object (gse, parent->se_obj,
+					    button->widget.id,
+					    button);
+
+  return button;
+}
+
 /*
  *
  * subview stuff
  *
  */
 
-static void subview_layout (skin_widget_t *w, int xo, int yo) {
+static void subview_layout (skin_widget_t *w) {
   subview_t  *s = (subview_t *) w;
   GList      *n;
 
-  s->widget.x  = se_eval_int (s->widget.xscript, s->widget.parent) + xo;
-  s->widget.y  = se_eval_int (s->widget.yscript, s->widget.parent) + yo;
-
-  if (s->pix) {
-    s->h_max     = se_eval_int (s->h_max_script, s->widget.parent);
-    s->v_max     = se_eval_int (s->v_max_script, s->widget.parent);
-  }
+  s->widget.x  = se_eval_int (s->widget.xscript, w->se_obj);
+  s->widget.y  = se_eval_int (s->widget.yscript, w->se_obj);
 
   /*
    * layout children
@@ -683,45 +835,55 @@ static void subview_layout (skin_widget_t *w, int xo, int yo) {
 
     w = (skin_widget_t *) n->data;
 
-    w->layout (w, s->widget.x, s->widget.y);
+    w->layout (w);
 
     n = g_list_next (n);
   }
 }
 
 
-static void subview_paint (skin_widget_t *w) {
+static void subview_paint (skin_widget_t *w, int parent_xo, int parent_yo) {
   subview_t *s = (subview_t *) w;
-  view_t    *view = s->widget.parent;
+  view_t    *view = w->view;
   GList      *n;
 
   if (s->pix) {
     gboolean finished;
     gint     xoff, yoff;
     
-    finished = 0; xoff = 0; yoff = 0;
+    if (s->halign && !strcasecmp (s->halign, "right")) 
+      xoff = w->parent->w - w->x;
+    else
+      xoff = w->x;
+
+    if (s->valign && !strcasecmp (s->valign, "bottom")) 
+      yoff = w->parent->h - w->y;
+    else
+      yoff = w->y;
+
+    xoff = w->x;
+    yoff = w->y;
+    finished = 0; 
     do {
       gdk_pixbuf_render_to_drawable (s->pix, view->bg, NULL, 0, 0,
-				     s->widget.x+xoff, 
-				     s->widget.y+yoff,
+				     xoff+parent_xo, 
+				     yoff+parent_yo,
 				     -1, -1, GDK_RGB_DITHER_MAX, 0, 0);
       
       my_gdk_pixbuf_render_threshold_alpha (s->pix, view->mask,
 					    0, 0, 
-					    s->widget.x+xoff, 
-					    s->widget.y+yoff,
+					    xoff+parent_xo, 
+					    yoff+parent_yo,
 					    -1, -1, 1);
       if (s->tiled) {
 	if (s->halign && !strcasecmp (s->halign, "stretch")) {
 	  xoff += gdk_pixbuf_get_width (s->pix);
 	  
-	  if ((xoff+s->widget.x) > s->h_max)
-	    finished = TRUE;
+	  finished = (xoff > (w->parent->w - w->x)); 
 	} else if (s->valign && !strcasecmp (s->valign, "stretch")) {
 	  yoff += gdk_pixbuf_get_height (s->pix);
 	  
-	  if ((yoff+s->widget.y) > s->v_max) 
-	    finished = TRUE;
+	  finished = (yoff > (w->parent->h - w->y)); 
 	} else
 	  finished = TRUE;
       } else
@@ -735,11 +897,11 @@ static void subview_paint (skin_widget_t *w) {
 
   n = g_list_first (s->widgets);
   while (n) {
-    skin_widget_t *w;
+    skin_widget_t *cw;
 
-    w = (skin_widget_t *) n->data;
+    cw = (skin_widget_t *) n->data;
 
-    w->paint (w);
+    cw->paint (cw, parent_xo + s->widget.x, parent_yo + s->widget.y);
 
     n = g_list_next (n);
   }
@@ -750,17 +912,19 @@ static void subview_handle_button_release (skin_widget_t *w, int px, int py) {
   subview_t *s = (subview_t *) w;
   GList     *n;
 
+  px -= w->x;
+  py -= w->y;
+
   n = g_list_first (s->widgets);
   while (n) {
-    skin_widget_t *w;
+    skin_widget_t *cw;
 
-    w = (skin_widget_t *) n->data;
+    cw = (skin_widget_t *) n->data;
 
-    w->handle_button_release (w, px, py);
+    cw->handle_button_release (cw, px, py);
 
     n = g_list_next (n);
   }
-  return FALSE;
 }
 
 static gboolean subview_handle_button_press (skin_widget_t *w, int px, int py) {
@@ -768,13 +932,16 @@ static gboolean subview_handle_button_press (skin_widget_t *w, int px, int py) {
   subview_t *s = (subview_t *) w;
   GList     *n;
 
+  px -= w->x;
+  py -= w->y;
+
   n = g_list_first (s->widgets);
   while (n) {
-    skin_widget_t *w;
+    skin_widget_t *cw;
 
-    w = (skin_widget_t *) n->data;
+    cw = (skin_widget_t *) n->data;
 
-    if (w->handle_button_press (w, px, py))
+    if (cw->handle_button_press (cw, px, py))
       return TRUE;
 
     n = g_list_next (n);
@@ -786,26 +953,50 @@ static void subview_handle_motion (skin_widget_t *w, int px, int py) {
   subview_t *s = (subview_t *) w;
   GList     *n;
 
+  px -= w->x;
+  py -= w->y;
+
   n = g_list_first (s->widgets);
   while (n) {
-    skin_widget_t *w;
+    skin_widget_t *cw;
 
-    w = (skin_widget_t *) n->data;
+    cw = (skin_widget_t *) n->data;
 
-    w->handle_motion (w, px, py);
+    cw->handle_motion (cw, px, py);
 
     n = g_list_next (n);
   }
 }
 
-static subview_t *create_subview (xml_node_t *xsubview, view_t *view) {
+static int subview_prop_int_cb (void *subview_gen, char *prop, int *num) {
+
+  subview_t *subview = (subview_t *) subview_gen;
+
+  if (!strcasecmp (prop, "left")) {
+    subview->widget.x  = se_eval_int (subview->widget.xscript, subview->widget.se_obj);
+    *num = subview->widget.x;
+    return 1;
+  } else if (!strcasecmp (prop, "top")) {
+    subview->widget.y  = se_eval_int (subview->widget.yscript, subview->widget.se_obj);
+    *num = subview->widget.y;
+    return 1;
+  }
+
+  logprintf ("skin_window: looking for %s in subview %s\n",
+	     prop, subview->widget.id);
+
+  return 0;
+}
+
+static subview_t *create_subview (xml_node_t *xsubview, skin_widget_t *parent) {
 
   subview_t  *subview;
   xml_node_t *node;
 
   subview = (subview_t *) malloc (sizeof (subview_t));
 
-  subview->widget.parent  = view;
+  subview->widget.parent  = parent;
+  subview->widget.view    = parent->view;
   subview->widget.id      = xml_parser_get_property     (xsubview, "id");
   subview->widget.zIndex  = xml_parser_get_property_int (xsubview, "zIndex", 0);
 
@@ -821,11 +1012,15 @@ static subview_t *create_subview (xml_node_t *xsubview, view_t *view) {
   subview->tiled   = xml_parser_get_property_bool(xsubview, "backgroundTiled", FALSE);
   subview->halign  = xml_parser_get_property     (xsubview, "horizontalAlignment");
   subview->valign  = xml_parser_get_property     (xsubview, "verticalAlignment");
-  subview->h_max_script = xml_parser_get_property(xsubview, "horizontalLimit");
-  subview->v_max_script = xml_parser_get_property(xsubview, "verticalLimit");
 
-  subview->pix = load_pix_trans (xsubview, "backgroundImage", view->basename);
+  subview->pix = load_pix_trans (xsubview, "backgroundImage", parent->view->basename);
 
+  if (!subview->widget.id)
+    subview->widget.id = unique_name("subview");
+
+  subview->widget.se_obj = se_create_object (gse, parent->se_obj,
+					     subview->widget.id,
+					     subview);
   subview->widgets = NULL;
 
   node = xsubview->child;
@@ -833,18 +1028,29 @@ static subview_t *create_subview (xml_node_t *xsubview, view_t *view) {
 
     if (!strcasecmp (node->name, "subview")) {
 
-      subview_t *subview;
+      subview_t *s;
 
-      subview = create_subview (node, view);
+      s = create_subview (node, &subview->widget);
 
-      subview->widgets = g_list_insert_sorted (subview->widgets, subview, widgetzcomp);
+      subview->widgets = g_list_insert_sorted (subview->widgets, s, widgetzcomp);
     } else if (!strcasecmp (node->name, "buttongroup")) {
       buttongroup_t *buttongroup;
 
-      buttongroup = create_buttongroup (node, view);
+      buttongroup = create_buttongroup (node, &subview->widget);
 
       subview->widgets = g_list_insert_sorted (subview->widgets, buttongroup, widgetzcomp);
       
+    } else if (!strcasecmp (node->name, "button") 
+	       || !strcasecmp (node->name, "pausebutton")) {
+      button_t *button;
+
+      button = create_button (node, &subview->widget);
+
+      subview->widgets = g_list_insert_sorted (subview->widgets, button, widgetzcomp);
+    } else if (!strcasecmp (node->name, "video")) {
+      /*video_t *video =*/ widget_video_create (node, &subview->widget);
+    } else {
+      printf ("*** error, '%s' unknown node type\n", node->name);
     }
 
     node = node->next;
@@ -868,7 +1074,7 @@ static void paint_view (view_t *view) {
 
     w = (skin_widget_t *) n->data;
 
-    w->layout (w, 0, 0);
+    w->layout (w);
 
     n = g_list_next (n);
   }
@@ -883,7 +1089,7 @@ static void paint_view (view_t *view) {
 
     w = (skin_widget_t *) n->data;
 
-    w->paint (w);
+    w->paint (w, 0, 0);
 
     n = g_list_next (n);
   }
@@ -909,8 +1115,8 @@ static void view_motion_cb (GtkWidget *widget, GdkEventMotion *event,
     
     gdk_window_get_pointer (NULL, &mx, &my, &modmask);
     
-    newx = mx - view->x;
-    newy = my - view->y;
+    newx = mx - view->widget.x;
+    newy = my - view->widget.y;
     gdk_window_move (view->window->window, newx, newy);
 
   } else {
@@ -1018,8 +1224,8 @@ static gboolean view_button_press_cb (GtkWidget *widget, GdkEventButton *event,
       
       if (!handled) {
 	view->is_moving = TRUE;
-	view->x = event->x;
-	view->y = event->y;
+	view->widget.x = event->x;
+	view->widget.y = event->y;
       }
     }
   }
@@ -1027,20 +1233,31 @@ static gboolean view_button_press_cb (GtkWidget *widget, GdkEventButton *event,
   return TRUE;
 }
 
-static view_t *create_view (xml_node_t *xview, const char *basename,
+static view_t *create_view (theme_t *theme, xml_node_t *xview, const char *basename,
 			    int view_num) {
 
   xml_node_t *node;
   view_t     *view;
-
+  char       *scriptfile;
+  
   view = (view_t *) malloc (sizeof (view_t));
 
-  view->w         = xml_parser_get_property_int (xview, "width", 0);
-  view->h         = xml_parser_get_property_int (xview, "height", 0);
-  view->id        = xml_parser_get_property     (xview, "id");
+  view->widget.parent  = NULL;
+  view->widget.view    = view;
+  view->widget.zIndex  = 0;
+
+#if 0
+  view->widget.xscript = xml_parser_get_property     (xsubview, "left");
+  view->widget.yscript = xml_parser_get_property     (xsubview, "top");
+#endif
+
+  view->widget.w         = xml_parser_get_property_int (xview, "width", 0);
+  view->widget.h         = xml_parser_get_property_int (xview, "height", 0);
+  view->widget.id        = xml_parser_get_property     (xview, "id");
+
   view->is_moving = FALSE;
 
-  if (!view->w || !view->h) {
+  if (!view->widget.w || !view->widget.h) {
     free (view);
     return NULL;
   }
@@ -1048,33 +1265,57 @@ static view_t *create_view (xml_node_t *xview, const char *basename,
   view->basename = strdup (basename);
   view->widgets  = NULL;
 
-  node = xview->child;
-  while (node) {
+  /*
+   * create corresponding javascript object
+   */
 
-    if (!strcasecmp (node->name, "subview")) {
+  if (!view->widget.id)
+    view->widget.id = unique_name ("view");
+  view->widget.se_obj = se_create_object (gse, NULL, "view", view);
 
-      subview_t *subview;
+  se_prop_create_int (gse, view->widget.se_obj, "width", view->widget.w);
+  se_prop_create_int (gse, view->widget.se_obj, "height", view->widget.w);
+  
+  /*
+   * compile associated js file
+   */
 
-      subview = create_subview (node, view);
+  scriptfile = xml_parser_get_property  (xview, "scriptfile");
 
-      view->widgets = g_list_insert_sorted (view->widgets, subview, widgetzcomp);
-    } else if (!strcasecmp (node->name, "buttongroup")) {
-      buttongroup_t *buttongroup;
+  if (scriptfile) {
+    char  jsfile[MAXPATH];
+    char *js_code, *onload, *semi;
 
-      buttongroup = create_buttongroup (node, view);
+    snprintf (jsfile, MAXPATH, "%s/%s", 
+	      basename,
+	      scriptfile);
 
-      view->widgets = g_list_insert_sorted (view->widgets, buttongroup, widgetzcomp);
-      
+    semi = strchr (jsfile, ';');
+    if (semi)
+      *semi = 0;
+    
+    printf ("skin_window: loading js file %s\n", jsfile);
+
+    js_code = read_entire_file (jsfile);
+
+    printf ("skin_window: executing it...\n");
+
+    se_eval (gse, js_code, NULL, print_cb, NULL);
+
+    onload = xml_parser_get_property  (xview, "onload");
+    
+    if (onload) {
+      printf ("skin_window: queueing '%s' (onload)\n", onload);
+
+      theme->jobs = g_list_append (theme->jobs, onload);
     }
-
-    node = node->next;
   }
 
   view->window =  gtk_window_new(GTK_WINDOW_TOPLEVEL);
 
   gtk_widget_set_app_paintable (view->window, TRUE);
 
-  gtk_widget_set_usize (view->window, view->w, view->h);
+  gtk_widget_set_size_request (view->window, view->widget.w, view->widget.h);
 
   gtk_widget_add_events (view->window, GDK_POINTER_MOTION_MASK 
 			 | GDK_BUTTON_MOTION_MASK    
@@ -1083,24 +1324,49 @@ static view_t *create_view (xml_node_t *xview, const char *basename,
 			 | GDK_BUTTON_RELEASE_MASK 
 			 | GDK_KEY_PRESS_MASK);
 
-  g_signal_connect (G_OBJECT (view->window), "button_press_event",
+  g_signal_connect (G_OBJECT (view->window), "button-press-event",
 		    G_CALLBACK (view_button_press_cb), view);
-  g_signal_connect (G_OBJECT (view->window), "button_release_event",
+  g_signal_connect (G_OBJECT (view->window), "button-release-event",
 		    G_CALLBACK (view_button_release_cb), view);
-  g_signal_connect (G_OBJECT (view->window), "motion_notify_event",
+  g_signal_connect (G_OBJECT (view->window), "motion-notify-event",
 		    G_CALLBACK (view_motion_cb), view);
 
   gtk_widget_realize (view->window);
 
-  view->bg   = gdk_pixmap_new (view->window->window, view->w, view->h, -1);
-  view->mask = gdk_pixmap_new (view->window->window, view->w, view->h, 1);
+  view->bg   = gdk_pixmap_new (view->window->window, view->widget.w, view->widget.h, -1);
+  view->mask = gdk_pixmap_new (view->window->window, view->widget.w, view->widget.h, 1);
+
+  view->fixed = gtk_fixed_new ();
+  gtk_container_add (GTK_CONTAINER (view->window), view->fixed);
+  
+  node = xview->child;
+  while (node) {
+
+    if (!strcasecmp (node->name, "subview")) {
+
+      subview_t *subview;
+
+      subview = create_subview (node, &view->widget);
+
+      view->widgets = g_list_insert_sorted (view->widgets, subview, widgetzcomp);
+    } else if (!strcasecmp (node->name, "buttongroup")) {
+      buttongroup_t *buttongroup;
+
+      buttongroup = create_buttongroup (node, &view->widget);
+
+      view->widgets = g_list_insert_sorted (view->widgets, buttongroup, widgetzcomp);
+      
+    }
+
+    node = node->next;
+  }
 
   {
     GdkGC *gc;
     
     gc = gdk_gc_new (view->mask);
     
-    gdk_draw_rectangle (view->mask, gc, TRUE, 0, 0, view->w, view->h);
+    gdk_draw_rectangle (view->mask, gc, TRUE, 0, 0, view->widget.w, view->widget.h);
   }
 
   paint_view (view);
@@ -1115,39 +1381,93 @@ static view_t *create_view (xml_node_t *xview, const char *basename,
   /* if (!view_num) */
     gtk_widget_show_all (view->window);
 
+  if (gtv) {
+    /* FIXME: bad hack */
+    printf ("gtv is: %p\n", gtv);
+    gtk_widget_hide (gtv);
+    gtk_widget_show (gtv);
+  }
+
   return view;
 }
 
-static void create_theme (xml_node_t *theme, char *basename) {
+static int theme_prop_int_cb (void *theme_gen, char *prop, int *num) {
+
+  /* buttongroup_t *buttongroup = (buttongroup_t *) buttongroup_gen; */
+
+  printf ("skin_window: looking for %s in theme\n", prop);
+
+  return 0;
+}
+
+static JSBool load_pref_cb (JSContext *cx, JSObject *obj, uintN argc, 
+			    jsval *argv, jsval *rval) {
+
+  JSString *str;
+  
+  printf ("FIXME: load_pref_cb called, not implemented yet\n");
+
+  str = JS_NewString(cx, "false", 5);
+  
+  *rval = STRING_TO_JSVAL (str);
+
+  return JS_TRUE;
+}
+
+static theme_t *create_theme (xml_node_t *xtheme, char *basename) {
 
   xml_node_t *node;
   int         view_num;
+  theme_t    *theme;
 
-  node = theme->child; view_num = 0;
+  theme = malloc (sizeof (theme_t));
+
+  theme->views = NULL;
+  theme->jobs = NULL;
+
+  theme->se_obj = se_create_object (gse, NULL,
+				    "theme",
+				    theme);
+
+  se_defun (gse, theme->se_obj, "loadPreference", load_pref_cb, 0, 0,
+	    SE_GROUP_HIDDEN, NULL, NULL);
+
+  node = xtheme->child; view_num = 0;
   while (node) {
 
     if (!strcasecmp (node->name, "view")) {
-      create_view (node, basename, view_num++);
+      view_t *view;
+
+      view = create_view (theme, node, basename, view_num++);
+
+      theme->views = g_list_append (theme->views, view);
+
     }
 
     node = node->next;
   }
+
+  return theme;
 }
 
 void create_skin_window (void) {
 
   /* char       *skinfile = "/home/guenter/projects/video/gnome-xine/skins/plus/bionic_d.wms";  */
-  /* char       *skinfile = "/home/guenter/projects/video/gnome-xine/skins/xbox_live/xlive.wms";  */
-   /* char       *skinfile = "/home/guenter/projects/video/gnome-xine/skins/kenwood/kenwood.wms"; */
-  char       *skinfile = "/home/guenter/projects/video/gnome-xine/skins/media_center/skin.gxs";
+  char       *skinfile = "/home/guenter/projects/video/gnome-xine/skins/xbox_live/xbox.wms";  
+  /* char       *skinfile = "/home/guenter/projects/video/gnome-xine/skins/kenwood/kenwood.wms"; */
+  /* char       *skinfile = "/home/guenter/projects/video/gnome-xine/skins/media_center/skin.gxs"; */
   char       *xml_data;
   xml_node_t *tree, *node;
 
   printf ("skin_window: loading %s...\n", skinfile);
 
-  xml_data = read_entire_file_ascii (skinfile);
+  xml_data = read_entire_file (skinfile);
+  if (!xml_data) {
+    printf ("failed to open %s: %s\n", skinfile, strerror (errno));
+    abort ();
+  }
 
-  printf ("skin_window: ascii: %s\n", xml_data); 
+  /* printf ("skin_window: ascii: %s\n", xml_data);  */
 
   printf ("skin_window: parsing...\n");
 
@@ -1158,6 +1478,17 @@ void create_skin_window (void) {
     return;
   }
 
+  /* xml_parser_dump_tree (tree); */
+
+#if 0
+  printf ("skin_window: loading %s...\n", jsfile);
+
+  js_code = read_entire_file (jsfile);
+
+  printf ("skin_window: javascript code:\n---------------------------------\n%s\n---------------------\n",
+	  js_code);
+#endif
+
   node = tree;
   while (node) {
 
@@ -1177,3 +1508,17 @@ void create_skin_window (void) {
   }
 
 }
+
+void window_toolbar_toggle (void)
+{
+}
+
+void window_toolbar_reset (void)
+{
+}
+
+void window_toolbar_restore (void)
+{
+}
+
+#endif /* EXP_STUFF */
