@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003 the xine project
+ * Copyright (C) 2003-2005 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: snapshot.c,v 1.8 2004/11/26 18:57:12 dsalt Exp $
+ * $Id: snapshot.c,v 1.10 2005/01/05 20:35:22 dsalt Exp $
  *
  * snapshot implementation
  *
@@ -72,7 +72,7 @@ static uint8_t *yuy2torgb (const uint8_t *data,
     *ptr++ = clip_8bit ((y - 3918 * u - 8130 * v) / 10000);	/* G */
     *ptr++ = clip_8bit ((y + 20172 * u) / 10000);		/* B */
 
-    y = (ptr[2] - 16) * 11644;
+    y = (data[2] - 16) * 11644;
     data += 4;
 
     *ptr++ = clip_8bit ((y + 15960 * v) / 10000);		/* R */
