@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003 the xine project
+ * Copyright (C) 2003-2005 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: snapshot.h,v 1.1 2003/02/28 00:07:00 guenter Exp $
+ * $Id: snapshot.h,v 1.3 2005/01/03 22:06:04 dsalt Exp $
  *
  * gxine snapshot function
  */
@@ -23,6 +23,9 @@
 #ifndef HAVE_SNAPSHOT_H
 #define HAVE_SNAPSHOT_H
 
-void make_snapshot (char *fname) ;
+void make_snapshot (char *fname, int scale, int blend);
+  /* -ve == from dbox, 0 == unscaled, +ve == scaled */
+
+void snapshot_init (void);
 
 #endif
