@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: stream_info.c,v 1.12 2004/12/02 02:38:32 dsalt Exp $
+ * $Id: stream_info.c,v 1.13 2004/12/22 03:37:32 dsalt Exp $
  *
  * stream_info/metadata display dialog implementation
  */
@@ -177,20 +177,19 @@ void stream_info_init (void) {
   gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tree_view), TRUE);
 
   cell = gtk_cell_renderer_text_new ();
-  column = gtk_tree_view_column_new_with_attributes (_("Field"),
-						     cell,
-						     "text", 0,
-						     NULL);
+  column = gtk_tree_view_column_new_with_attributes (_("Field"), cell,
+						     "text", 0, NULL);
+  gtk_tree_view_column_set_resizable (GTK_TREE_VIEW_COLUMN(column), TRUE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view),
 			       GTK_TREE_VIEW_COLUMN (column));
 
   column = gtk_tree_view_column_new_with_attributes (_("Data from stream"),
-						     cell,
-						     "text", 1,
-						     NULL);
+						     cell, "text", 1, NULL);
+  gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN(column),
+				   GTK_TREE_VIEW_COLUMN_AUTOSIZE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view),
 			       GTK_TREE_VIEW_COLUMN (column));
-  
+
   scrolled_window = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
 				  GTK_POLICY_AUTOMATIC, 
