@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2004 the xine-project
+ * Copyright (C) 2001-2005 the xine-project
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -16,7 +16,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
  *
- * $Id: utils.c,v 1.40 2004/12/17 00:33:00 dsalt Exp $
+ * $Id: utils.c,v 1.50.2.3 2005/07/06 19:13:45 dsalt Exp $
  *
  * needful things, and various UI bits:
  * - button registry and update (state display etc.)
@@ -47,6 +47,7 @@
 #include "http.h"
 #include "player.h"
 #include "playlist.h"
+#include "gtkvideo.h"
 
 void ensure_path_exists (char *path) {
 
@@ -126,20 +127,6 @@ char *read_entire_file (const char *mrl, int *file_size) {
   return buf;
 }
 
-GdkPixbuf *load_icon (const char *filename) {
-
-  GdkPixbuf *pix;
-  gchar     *pathname;
-
-  pathname = g_strconcat (pixmapdir, "/", filename, NULL);
-
-  pix = gdk_pixbuf_new_from_file (pathname, NULL);
-
-  g_free (pathname);
-
-  return pix;
-}
-
 static void response_cb (GtkWidget *dialog, gint response, gpointer data) {
   char *msg;
   
@@ -154,15 +141,22 @@ void v_display_message (const gchar *title, GtkMessageType type,
 {
   GtkWidget *dialog;
   gchar     *msg;
+  gboolean   modal = (fmt == NULL);
+
+  if (modal)
+    fmt = va_arg (ap, const gchar *);
 
   msg = g_strdup_vprintf (fmt, ap);
   va_end (ap);
 
   dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT, type,
-				   GTK_BUTTONS_CLOSE, msg);
+				   GTK_BUTTONS_CLOSE, "%s", msg);
   gtk_window_set_title (GTK_WINDOW (dialog), title);
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
 
+  if (modal)
+    gtk_window_set_modal (GTK_WINDOW(dialog), TRUE);
+
   g_signal_connect (G_OBJECT (dialog), "response",
 		    G_CALLBACK (response_cb), NULL);
   g_object_set_data (G_OBJECT (dialog), "msg", msg);
@@ -382,7 +376,7 @@ void utils_init (void) {
 		    G_CALLBACK (cancel_cb),
 		    NULL);
 
-  g_signal_connect( GTK_OBJECT (file_dlg), "delete_event",
+  g_signal_connect( GTK_OBJECT (file_dlg), "delete-event",
 		    G_CALLBACK (cancel_cb), NULL );
 }
 
@@ -420,6 +414,7 @@ char *asreprintf (char **str, const char *fmt, ...)
 
   va_start (ap, fmt);
   ret = g_strdup_vprintf (fmt, ap);
+  va_end (ap);
   free (*str);
   return *str = ret;
 }
@@ -564,7 +559,7 @@ static const ui_adjustable_t ranges[] = {
   [Control_VOLUME]		= {    50,       0,    100,    1,    10, 1, "audio.volume.mixer_volume", XINE_PARAM_AUDIO_VOLUME },
   [Control_COMPRESSOR]		= {   100,     100,   1000,   10,   100, 1, "gui.ao_compressor", XINE_PARAM_AUDIO_COMPR_LEVEL },
   [Control_AMPLIFIER]		= {   100,       0,    200,    1,    10, 1, "gui.ao_amplifier",  XINE_PARAM_AUDIO_AMP_LEVEL },
-  [Control_AV_SYNC]		= {     0, -180000, 180000, 1000, 10000, 1, NULL,		 XINE_PARAM_AV_OFFSET },
+  [Control_AV_SYNC]		= {     0, -180000, 180000, 1000, 10000, 1, "gui.av_sync",	 XINE_PARAM_AV_OFFSET },
   [Control_HUE]			= { 32768,       0,  65535,  100,  1000, 1, "gui.vo_hue",	 XINE_PARAM_VO_HUE },
   [Control_SATURATION]		= { 32768,       0,  65535,  100,  1000, 1, "gui.vo_saturation", XINE_PARAM_VO_SATURATION },
   [Control_CONTRAST]		= { 32768,       0,  65535,  100,  1000, 1, "gui.vo_contrast",   XINE_PARAM_VO_CONTRAST },
@@ -593,8 +588,9 @@ static void ui_adjustment_value_changed_cb (GtkAdjustment *adj,
   if (info->param)
     xine_set_param (stream, info->param, adj->value);
 
-  if (info->param != XINE_PARAM_AUDIO_VOLUME &&
-      info->setting && xine_config_lookup_entry (xine, info->setting, &entry))
+  if (info->param == XINE_PARAM_AUDIO_VOLUME)
+    return;
+  if (info->setting && xine_config_lookup_entry (xine, info->setting, &entry))
   {
     entry.num_value = adj->value;
     xine_config_update_entry (xine, &entry);
@@ -642,8 +638,10 @@ void ui_set_control_button (control_button_t item, gboolean state)
   GSList *btn;
   g_return_if_fail (item >= Control_PlayerButtons && item < Control_Buttons);
   no_recursion = TRUE;
-  foreach_glist (btn, c_buttons[item])
-    g_object_set (G_OBJECT(btn->data), "active", state, NULL);
+  if (c_buttons[item] &&
+      (GTK_TOGGLE_BUTTON(c_buttons[item]->data))->active != state)
+    foreach_glist (btn, c_buttons[item])
+      g_object_set (G_OBJECT(btn->data), "active", state, NULL);
   no_recursion = FALSE;
 }
 
@@ -653,7 +651,8 @@ void ui_set_control_adjustment (control_adjustment_t item, gdouble value)
 {
   g_return_if_fail (item >= 0 && item < Control_Adjustments);
   no_recursion = TRUE;
-  g_object_set (G_OBJECT(c_adjustments[item]), "value", value, NULL);
+  if ((GTK_ADJUSTMENT(c_adjustments[item]))->value != value)
+    g_object_set (G_OBJECT(c_adjustments[item]), "value", value, NULL);
   no_recursion = FALSE;
 }
 
@@ -739,6 +738,25 @@ static JSBool js_toolbar_position (JSContext *cx, JSObject *obj, uintN argc,
   return JS_TRUE;
 }
 
+static void post_deinterlace_plugin_cb (void *data, xine_cfg_entry_t *cfg)
+{
+  if (gtv)
+    gtk_video_set_tvtime (GTK_VIDEO(gtv), cfg->str_value);
+}
+
+static void post_plugins_video_cb (void *data, xine_cfg_entry_t *cfg)
+{
+  if (gtv)
+    gtk_video_set_post_plugins_video (GTK_VIDEO(gtv), cfg->str_value);
+}
+
+static void post_plugins_audio_cb (void *data, xine_cfg_entry_t *cfg)
+{
+  if (gtv)
+    gtk_video_set_post_plugins_audio (GTK_VIDEO(gtv), cfg->str_value,
+				      audio_port);
+}
+
 void ui_preferences_register (xine_t *this)
 {
   static char *experience_labels[] = {
@@ -750,16 +768,17 @@ void ui_preferences_register (xine_t *this)
     N_("Top, visible"), N_("Bottom, visible"),
     NULL
   };
+  const char **vis_labels, *const *vis_src;
+  int i;
 
   /* Register the experience level setting */
   xine_config_register_enum
     (this, "gui.experience_level", 0, experience_labels,
-     _("display of configuration settings"),
+     _("Display of configuration settings"),
      _("Controls whether more advanced configuration settings are shown."),
      0, NULL, CONFIG_DATA_NONE);
 
   /* Register a few audio/video config items */
-  int i;
   for (i = 0; i < sizeof (ranges) / sizeof (ranges[0]); ++i)
   {
     if (ranges[i].setting && !strncmp (ranges[i].setting, "gui.", 4))
@@ -770,15 +789,56 @@ void ui_preferences_register (xine_t *this)
   }
 
   /* Register some front-end widget options */
+  xine_config_register_string
+    (this, "gui.post_plugins.deinterlace",
+     "tvtime:method=LinearBlend,cheap_mode=1,pulldown=none,use_progressive_frame_flag=1",
+     _("Deinterlace plugin name and parameters"),
+     _("Format: tvtime:arg=value,arg=value,..."),
+     40, post_deinterlace_plugin_cb, CONFIG_DATA_NONE);
+
   xine_config_register_bool
-    (this, "gui.deinterlace_by_default", 0,
-     _("enable deinterlacing by default"),
-     _("Deinterlace plugin will be enabled on startup. Progressive streams are automatically detected with no performance penalty."),
-     0, NULL, NULL);
+    (this, "gui.post_plugins.deinterlace_enable", 1,
+     _("Enable deinterlacing at startup"), NULL,
+     30, NULL, NULL);
+
+  xine_config_register_string
+    (this, "gui.post_plugins.video", "",
+     _("Video post-processing plugins' names and parameters"),
+     _("Format: plugin:arg=value,arg=value,...;plugin:..."),
+     40, post_plugins_video_cb, CONFIG_DATA_NONE);
+
+  xine_config_register_bool
+    (this, "gui.post_plugins.video_enable", 0,
+     _("Enable video post-processing at startup"), NULL,
+     30, NULL, NULL);
+
+  xine_config_register_string
+    (this, "gui.post_plugins.audio", "",
+     _("Audio post-processing plugins' names and parameters"),
+     _("Format: plugin:arg=value,arg=value,...;plugin:..."),
+     40, post_plugins_audio_cb, CONFIG_DATA_NONE);
+
+  xine_config_register_bool
+    (this, "gui.post_plugins.audio_enable", 0,
+     _("Enable audio post-processing at startup"), NULL,
+     30, NULL, NULL);
+
+  vis_src = xine_list_post_plugins_typed
+	      (xine, XINE_POST_TYPE_AUDIO_VISUALIZATION);
+  for (i = 0; vis_src[i]; ++i) /**/;
+  vis_labels = calloc (i + 2, sizeof (char *));
+  vis_labels[0] = N_("None"); /* translate for display, not for config file */
+  memcpy (vis_labels + 1, vis_src, (i + 1) * sizeof (char *));
+
+  xine_config_register_enum
+    (this, "gui.post_plugins.audio_visualisation", 1, vis_labels,
+     _("Default audio visualisation plugin"),
+     _("Post-plugin to be used when playing streams without video"),
+     30, NULL, NULL);
 
   xine_config_register_enum
     (this, "gui.fullscreen_toolbar", 1, tbar_pos_labels,
-     _("default position & visibility of the full-screen toolbar"),
+     _("Default position & visibility of the full-screen toolbar"),
      NULL, 0, NULL, NULL);
 }
 
