@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2004 the xine-project
+ * Copyright (C) 2001-2005 the xine-project
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -16,7 +16,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
  *
- * $Id: utils.h,v 1.24 2004/12/17 00:33:00 dsalt Exp $
+ * $Id: utils.h,v 1.26.2.2 2005/07/06 19:13:45 dsalt Exp $
  *
  * needful things
  */
@@ -39,9 +39,9 @@ void ensure_path_exists (char *path) ;
  */
 char *read_entire_file (const char *mrl, int *file_size);
 
-/* load an image from the system-wide gxine image dir */
-GdkPixbuf *load_icon (const char *filename);
-
+/* display message boxes
+ * if fmt==NULL, first vararg is format string and window is modal
+ */
 void v_display_message (const gchar *title, GtkMessageType,
 			const gchar *fmt, va_list);
 void display_error (const gchar *title, const gchar *fmt, ...) ;
@@ -146,6 +146,7 @@ void window_toolbar_show (gboolean);
 void window_toolbar_position (gboolean top);
 void window_toolbar_reset (void);
 void window_toolbar_restore (void);
+void window_check_vis (gboolean force);
 
 void ui_init (void);
 
