@@ -16,7 +16,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
  *
- * $Id: wizards.c,v 1.28 2004/12/02 02:38:32 dsalt Exp $
+ * $Id: wizards.c,v 1.31 2005/02/01 21:56:42 dsalt Exp $
  *
  * stuff to run on first startup / setup wizards
  */
@@ -41,9 +41,6 @@
 #include "utils.h"
 #include "desktop_integration.h"
 
-#include "ok.xpm"
-#include "fail.xpm"
-
 typedef struct {
   char *msg;
   char *explanation;
@@ -65,15 +62,8 @@ static GtkWidget *do_health_check (GtkWidget *dlg, GtkWidget *headline) {
   xine_health_check_t  hc;
   xine_health_check_t *results;
   int                  check;
-  GdkPixmap           *ok_pix, *fail_pix;
   hc_msg_t            *msg;
 
-  ok_pix   = gdk_pixmap_create_from_xpm_d (dlg->window, 
-					   NULL, NULL, ok_xpm);
-  fail_pix = gdk_pixmap_create_from_xpm_d (dlg->window, 
-					   NULL, NULL, fail_xpm);
-  
-
   vbox = gtk_vbox_new (0, 0);
 
   table = gtk_table_new (1, 1, FALSE);
@@ -119,7 +109,7 @@ static GtkWidget *do_health_check (GtkWidget *dlg, GtkWidget *headline) {
     switch (results->status) {
     case XINE_HEALTH_CHECK_UNSUPPORTED:
     case XINE_HEALTH_CHECK_OK:
-      l = gtk_pixmap_new (ok_pix, NULL);
+      l = gtk_image_new_from_stock (GTK_STOCK_YES, GTK_ICON_SIZE_BUTTON);
       break;
     case XINE_HEALTH_CHECK_FAIL:
       msg = malloc (sizeof (hc_msg_t));
@@ -127,13 +117,13 @@ static GtkWidget *do_health_check (GtkWidget *dlg, GtkWidget *headline) {
       msg->msg = results->msg;
       msg->explanation = results->explanation;
 
-      l = gtk_pixmap_new (fail_pix, NULL);
-
       b = gtk_button_new_with_label (_("Details"));
       gtk_signal_connect (GTK_OBJECT (b), "clicked",
 			  GTK_SIGNAL_FUNC (details_cb), msg);
       gtk_table_attach (GTK_TABLE (table), b, 2, 3, check, check+1, GTK_FILL,
 			GTK_SHRINK, 2, 5);
+
+      l = gtk_image_new_from_stock (GTK_STOCK_NO, GTK_ICON_SIZE_BUTTON);
       break;
     }
 
@@ -150,6 +140,7 @@ static GtkWidget *do_health_check (GtkWidget *dlg, GtkWidget *headline) {
   return vbox;
 }
 
+#ifdef USE_INTEGRATION_WIZARD
 static GtkWidget *cb_moz;
 static GtkWidget *cb_gnome;
 static GtkWidget *cb_kde;
@@ -268,6 +259,7 @@ static void desktop_integration (void) {
     di_register_mozilla();
   }
 }
+#endif
 
 #define RESPONSE_CLOSE 0
 #define RESPONSE_NEXT  1
@@ -296,11 +288,17 @@ static void response_cb (GtkDialog *dbox, int response, gpointer data)
 
 #define WIZARDS_LEVEL 6
 
-void run_wizards (gboolean requested) {
+void run_wizards (gboolean requested)
+{
+  static GtkWidget *dlg = NULL;
+  GtkWidget *headline, *sep, *content, *w, *hbox;
+  int b, state;
 
-  int        b;
-  GtkWidget *dlg, *headline, *sep, *content, *w, *hbox;
-  int        state;
+  if (dlg)
+  {
+    gtk_window_present (GTK_WINDOW(dlg));
+    return;
+  }
 
   if (requested
       || (xine_config_register_num (xine, "misc.wizards_shown", 0,
@@ -376,11 +374,8 @@ void run_wizards (gboolean requested) {
 
     gtk_main ();
 
-    if (b==RESPONSE_CLOSE) {
-      gtk_widget_hide (dlg);
-      return;
-    }
-
+    if (b == RESPONSE_CLOSE)
+      goto finished;
 
     state = 0;
     while (TRUE) {
@@ -397,14 +392,13 @@ void run_wizards (gboolean requested) {
 
 	gtk_main ();
 
-	if (b==RESPONSE_CLOSE) {
-	  gtk_widget_hide (dlg);
-	  return;
-	}
+	if (b == RESPONSE_CLOSE)
+	  goto finished;
 
 	state ++;
 	break;
 
+#ifdef USE_INTEGRATION_WIZARD
       case 1:
 	w = ask_integration_wizard (dlg, headline);
 
@@ -413,20 +407,18 @@ void run_wizards (gboolean requested) {
 
 	gtk_main ();
 
-	if (b==RESPONSE_CLOSE) {
-	  gtk_widget_hide (dlg);
-	  return;
-	}
-	
-	desktop_integration ();
-	  
+	if (b != RESPONSE_CLOSE)
+	  desktop_integration ();
+#endif
+
       default:
-	gtk_widget_hide (dlg);
-	return;
+        goto finished;
       }
     }
 
+finished:
     gtk_widget_hide (dlg);
+    dlg = NULL;
   }
 }
 
