@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- *  $Id: xmlparser.h,v 1.4 2003/04/19 06:09:06 guenter Exp $
+ *  $Id: xmlparser.h,v 1.5.2.1 2005/03/08 23:34:44 dsalt Exp $
  *
  */
 #ifndef XML_PARSER_H
@@ -60,6 +60,18 @@ int   xml_parser_get_property_int (xml_node_t *node, const char *name,
 int xml_parser_get_property_bool (xml_node_t *node, const char *name, 
 				  int def_value);
 
+/* for output:
+ * returns an escaped string (free() it when done)
+ * input must be in ASCII or UTF-8
+ */
+
+typedef enum {
+  XML_ESCAPE_NO_QUOTE,
+  XML_ESCAPE_SINGLE_QUOTE,
+  XML_ESCAPE_DOUBLE_QUOTE
+} xml_escape_quote_t;
+char *xml_escape_string (const char *s, xml_escape_quote_t quote_type);
+
 /* for debugging purposes: dump read-in xml tree in a nicely
  * indented fashion
  */
